
//
// Stanford2010 MOM Parser
// File: stanford2010parser.js
//
// PRODUCTION FUNCTIONS
// Functions are mainly independent in a sense that on the whole they are lacking optimization
// and may contain overlapping functionality.
// In order to reduce the code footprint some reformatting is adviced.
//
// THE MIT License
// https://opensource.org/licenses/MIT
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is furnished
// to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


/*
 * PRODUCTION
 * Returns the type of a JSON Object AS STRING
 */
function getJsonReturnType(object) {
	var stringConstructor = "test".constructor;
	var arrayConstructor = [].constructor;
	var objectConstructor = {}.constructor;
	if (object === null) {
        return 'null';
    }
    else if (object === undefined) {
        return 'undefined';
    }
    else if (object.constructor === stringConstructor) {
        return 'String';
    }
    else if (object.constructor === arrayConstructor) {
        return 'Array';
    }
    else if (object.constructor === objectConstructor) {
        return 'Object';
    }
    else {
        return 'dont know';
    }
}	

/*
 * PRODUCTION
 * Returns requested object AS JSON OBJECT, if exists, from document-OperationalMonitoring-Machine
 */
function OPERATIONAL_MONITORING_MACHINE_getValue(momAsJSON, momKey) {
	var jsonObject = JSON.parse(momAsJSON);
	var momValue = jsonObject.payload['#document'].OperationalMonitoring.Machine[momKey];
	if (getJsonReturnType(momValue) === 'Object') {
		//console.log(momKey + ' is an Object');
		momValue = jsonObject.payload['#document'].OperationalMonitoring.Machine[momKey]['_'];
		//console.log(momKey + ': ' + momValue);
		return momValue;
	} else if (getJsonReturnType(momValue) === 'undefined') {
		//console.log(momKey + ': -');
		return '-';
	} else {
		//console.log(momKey + ': ' + momValue);
		return momValue;
	}
}

/*
 * PRODUCTION
 * Returns machine work time object type AS STRING
 * Possible return values UNDEFINED, INDIVIDUAL, COMBINED 
 */
function OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime) {
		//console.log('COMBINED');
		return 'COMBINED';
	} else if (jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime) {
		//console.log('INDIVIDUAL');
		return 'INDIVIDUAL';
	} else {
		return 'UNDEFINED';
	}
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns operator Name as STRING
 */
function getOperatorName(momAsJSON, operatorKey) {
	
	var operatorDefinition = OPERATIONAL_MONITORING_MACHINE_getWhateverObject(momAsJSON, 'OperatorDefinition');
	
	var operatorFirstName = '';
	
	if (getJsonReturnType(operatorDefinition) === 'Object') {
		if (operatorDefinition.OperatorKey === operatorKey) {
			//console.log(operatorDefinition);
			if (operatorDefinition.ContactInformation.FirstName) {
				operatorFirstName = operatorDefinition.ContactInformation.FirstName;
			} else if (operatorDefinition.OperatorUserID) {
				if (getJsonReturnType(operatorDefinition.OperatorUserID) === 'String') {
					operatorFirstName = operatorDefinition.OperatorUserID;
				}
				if (getJsonReturnType(operatorDefinition.OperatorUserID) === 'Object') {
					operatorFirstName = operatorDefinition.OperatorUserID['_'];
				}
			} else {
				operatorFirstName = '';
			}
		}
	} else if (getJsonReturnType(operatorDefinition) === 'Array'){ 
		var operatorDefinitionObject;
		for (var operatorDefinitionObjectIndex in operatorDefinition) {
			operatorDefinitionObject = operatorDefinition[operatorDefinitionObjectIndex];
			if (operatorDefinitionObject.OperatorKey === operatorKey) {
				//console.log(operatorDefinitionObject);
				if (operatorDefinitionObject.ContactInformation) {
					if (operatorDefinitionObject.ContactInformation.FirstName) {
						//console.log('using first name');
						operatorFirstName = operatorDefinitionObject.ContactInformation.FirstName;
					}
				} else if (operatorDefinitionObject.OperatorUserID) {
					//console.log('using operator user id');
					if (getJsonReturnType(operatorDefinitionObject.OperatorUserID) === 'String') {
						//console.log('is string');
						operatorFirstName = operatorDefinitionObject.OperatorUserID;
					}
					if (getJsonReturnType(operatorDefinitionObject.OperatorUserID) === 'Object') {
						//console.log('is object');
						operatorFirstName = operatorDefinitionObject.OperatorUserID['_'];
					}
				} else {
					operatorFirstName = '';
				}
			}
		}
	}
	
	//console.log(operatorFirstName);
	
	if (operatorFirstName.length === 0) {
		operatorFirstName = 'NaN';
	}
	
	operatorFirstName = operatorFirstName.charAt(0).toUpperCase() + operatorFirstName.slice(1);
	
	return operatorFirstName;
	
}
 
/*
 * PRODUCTION
 * Returns operator keys AS ARRAY
 */
function OPERATIONAL_MONITORING_MACHINE_getOperatorKeys(momAsJSON) {
	var operatorKeys = [];
	//Get OperatorDefinition Object
	var operatorDefinition = OPERATIONAL_MONITORING_MACHINE_getWhateverObject(momAsJSON, 'OperatorDefinition');
	//OperatorDefinition Object Type
	if (getJsonReturnType(operatorDefinition) === 'Object') {
		operatorKeys.push(operatorDefinition.OperatorKey);
	} else if (getJsonReturnType(operatorDefinition) === 'Array'){
		var operatorDefinitionObject;
		for (var operatorDefinitionObjectIndex in operatorDefinition) {
			operatorDefinitionObject = operatorDefinition[operatorDefinitionObjectIndex];
			operatorKeys.push(operatorDefinitionObject.OperatorKey);
			//console.log(operatorDefinitionObject.OperatorKey);
		}
	} else {
		operatorKeys.push('');
	}
	return operatorKeys;
	
}

/*
 * PRODUCTION
 * Returns machine category (harvester, forwarder) AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_CATEGORY_getValue(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var momValue = jsonObject.payload['#document'].OperationalMonitoring.Machine['$'].machineCategory;
	//console.log(momKey + ': ' + momValue);
	return momValue;
}

/*
 * PRODUCTION
 * Returns all the timestamps (start and end times) in machine work time objects in milliseconds AS ARRAY
 */
function getMachineWorktimeTimestamps(momAsJSON) {

	var timestampMSArray = [];
	var jsonObject = JSON.parse(momAsJSON);
	var startTime;
	var endTime;
	var worktimeObjects;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		
		worktimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		if (getJsonReturnType(worktimeObjects) === 'Object') {
			timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects.MonitoringStartTime));
			timestampMSArray.push(getDateTimeInMilliseconds(getDateTimePlusSecondsAsDateTime(worktimeObjects.MonitoringStartTime, worktimeObjects.MonitoringTimeLength)));
		}
		if (getJsonReturnType(worktimeObjects) === 'Array') {
			for (var worktimeObject in worktimeObjects) {
				timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects[worktimeObject].MonitoringStartTime));
				timestampMSArray.push(getDateTimeInMilliseconds(getDateTimePlusSecondsAsDateTime(worktimeObjects[worktimeObject].MonitoringStartTime, worktimeObjects[worktimeObject].MonitoringTimeLength)));
			}
		}
			
	} else if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		
		worktimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		if (getJsonReturnType(worktimeObjects) === 'Object') {
			timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects.StartTime));
			timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects.CombinedEndTime));
		}
		if (getJsonReturnType(worktimeObjects) === 'Array') {
			for (var worktimeObject in worktimeObjects) {
				timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects[worktimeObject].StartTime));
				timestampMSArray.push(getDateTimeInMilliseconds(worktimeObjects[worktimeObject].CombinedEndTime));
			}
		}
	}
	
	return timestampMSArray;
	
}

/*
 * PRODUCTION
 * Returns start time on site AS DATETIME FORMATTED STRING
 */
function getMachineWorkTimeStartTime(momAsJSON) {
	var startTimeInMilliseconds =  Math.min(...getMachineWorktimeTimestamps(momAsJSON));
	//console.log('MIN TIMESTAMP IN MS: ' + startTimeInMilliseconds);
	//console.log('MIN TIMESTAMP IN DATETIME: ' + getMillisecondsAsDateTime(startTimeInMilliseconds));
	return getMillisecondsAsDateTime(startTimeInMilliseconds);
}

/*
 * PRODUCTION
 * Returns end time on site AS DATETIME FORMATTED STRING
 */
function getMachineWorkTimeEndTime(momAsJSON) {
	var endTimeInMilliseconds = Math.max(...getMachineWorktimeTimestamps(momAsJSON));
	//console.log('MAX TIMESTAMP IN MS: ' + endTimeInMilliseconds);
	//console.log('MAX TIMESTAMP IN DATETIME: ' + getMillisecondsAsDateTime(endTimeInMilliseconds));
	return getMillisecondsAsDateTime(endTimeInMilliseconds);
}

/*
 * PRODUCTION
 * Returns logging organization business name AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_OBJECT_DEFINITION_getLoggingOrganisation(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var loggingOrganisation = '';
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition) {
		if (jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition.LoggingOrganisation) {
			if (jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition.LoggingOrganisation.ContactInformation.BusinessName) {
				loggingOrganisation = jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition.LoggingOrganisation.ContactInformation.BusinessName;
			}
		} else {
			loggingOrganisation = jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition.ObjectName;
		}
	}
	return loggingOrganisation;
}

/*
 * PRODUCTION
 * Returns logging form AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_OBJECT_DEFINITION_getLoggingForm(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var loggingForm = '';
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition) {
		loggingForm = jsonObject.payload['#document'].OperationalMonitoring.Machine.ObjectDefinition.LoggingForm;
	}
	return loggingForm;
}

/*
 * PRODUCTION
 * Returns monitoring settings (filter times) object type AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_getMonitoringSettingsReturnType(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var momObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.MonitoringSettings;
	return getJsonReturnType(momObject);
}

/*
 * PRODUCTION
 * Returns monitoring settings (filter times) object AS JSON OBJECT
 */
function OPERATIONAL_MONITORING_MACHINE_getMonitoringSettings(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var momValue = jsonObject.payload['#document'].OperationalMonitoring.Machine.MonitoringSettings;
	//console.log(momKey + ': ' + momValue);
	return momValue;
}

/*
 * PRODUCTION
 * Returns FilterTimeDown from monitoring settings (filter times) object AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_MONITORING_SETTINGS_getMonitoringFilterTimeDown(monitoringSettingsObject) {
	return monitoringSettingsObject.MonitoringFilterTimeDown;
}

/*
 * PRODUCTION
 * Returns FilterTimeRun from monitoring sttings (filter times) object AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_MONITORING_SETTINGS_getMonitoringFilterTimeRun(monitoringSettingsObject) {
	return monitoringSettingsObject.MonitoringFilterTimeRun;
}

/*
 * PRODUCTION
 * Returns FilterTimeMinimum from monitoring settings (filter times) object AS STRING
 */
function OPERATIONAL_MONITORING_MACHINE_MONITORING_SETTINGS_getMonitoringFilterTimeMinimum(monitoringSettingsObject) {
	return monitoringSettingsObject.MonitoringFilterTimeMinimum;
}

/*
 * PRODUCTION
 * Returns operator specific sum of machine driven distance from machine work time objects AS INTEGER
 */
function getSumOfDrivenDistance(momAsJSON, operatorKey) {
	
	var machineWorkTimeObjects;
	var machineWorkTimeObject;
	var jsonObject = JSON.parse(momAsJSON);
	var sumOfDrivenDistance = 0;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') { 
		if (operatorKey === machineWorkTimeObjects.OperatorKey) {
			if (machineWorkTimeObjects.OtherMachineData) {
				sumOfDrivenDistance = parseInt(machineWorkTimeObjects.OtherMachineData.DrivenDistance);
			}
		}
	}
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (objectIndex in machineWorkTimeObjects) {
			if (operatorKey === machineWorkTimeObjects[objectIndex].OperatorKey) {
				if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
					sumOfDrivenDistance += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.DrivenDistance);
				}
			}
		}
	}
	
	return sumOfDrivenDistance;
	
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns operator specific sum of fuel consumption AS INTEGER
 * Filter values:	Processing
 *					Terrain travel
 *				    Other work
 *					null (returns total fuel consumption)
 */
function getSumOfFuelConsumption(momAsJSON, operatorKey, filter) {

	
	var jsonObject = JSON.parse(momAsJSON);
	var machineWorkTimeObjects;
	var sumOfFuelConsumption = 0;
	var workTimeObjectType;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		workTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		workTimeObjectType = 'COMBINED';
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		
		if (operatorKey === machineWorkTimeObjects.OperatorKey) {
			
			if (machineWorkTimeObjects.OtherMachineData) {
				if (filter) {
					if (workTimeObjectType === 'INDIVIDUAL') {
						if (machineWorkTimeObjects.IndividualMachineRunTimeCategory === filter) {
							sumOfFuelConsumption = parseInt(machineWorkTimeObjects.OtherMachineData.FuelConsumption);
						}
					}
					//if (workTimeObjectType === 'COMBINED') {
					//	sumOfFuelConsumption = parseInt(machineWorkTimeObjects.OtherMachineData.FuelConsumption);
					//}
				} else {
					if (workTimeObjectType === 'INDIVIDUAL') {
						sumOfFuelConsumption = parseInt(machineWorkTimeObjects.OtherMachineData.FuelConsumption);
					}
					if (workTimeObjectType === 'COMBINED') {
						sumOfFuelConsumption = parseInt(machineWorkTimeObjects.OtherMachineData.FuelConsumption);
					}
				}
			}
			
		}
		
	}
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (objectIndex in machineWorkTimeObjects) { 
			if (operatorKey === machineWorkTimeObjects[objectIndex].OperatorKey) {
			
				if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
					if (filter) {
						if (workTimeObjectType === 'INDIVIDUAL') {
							if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === filter) {
								sumOfFuelConsumption += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.FuelConsumption);
							}
						}
						//if (workTimeObjectType === 'COMBINED') {
						//	sumOfFuelConsumption += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.FuelConsumption);
						//}
					} else {
						if (workTimeObjectType === 'INDIVIDUAL') {
							sumOfFuelConsumption += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.FuelConsumption);
						}
						if (workTimeObjectType === 'COMBINED') {
							sumOfFuelConsumption += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.FuelConsumption);
						}
					}
				}
			
			}
		}
	}
	
	return sumOfFuelConsumption;
}
 
/*
 * DEVELOPMENT PRODUCTION
 * Returns operator specific sum of engine time in hours AS FLOAT
 */
function getSumOfEngineTime(momAsJSON, operatorKey) {

	
	var jsonObject = JSON.parse(momAsJSON);
	var machineWorkTimeObjects;
	var sumOfEngineTime = 0;
	var workTimeObjectType;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		workTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		workTimeObjectType = 'COMBINED';
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		
		if (operatorKey === machineWorkTimeObjects.OperatorKey) {
			
			if (machineWorkTimeObjects.OtherMachineData) {
				if (workTimeObjectType === 'INDIVIDUAL') {
					sumOfEngineTime = parseInt(machineWorkTimeObjects.OtherMachineData.EngineTime);
				}
				if (workTimeObjectType === 'COMBINED') {
					sumOfEngineTime = parseInt(machineWorkTimeObjects.OtherMachineData.EngineTime);
				}	
			}
			
		}
		
	}
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (objectIndex in machineWorkTimeObjects) { 
			if (operatorKey === machineWorkTimeObjects[objectIndex].OperatorKey) {
			
				if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
					
					if (workTimeObjectType === 'INDIVIDUAL') {
						sumOfEngineTime += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.EngineTime);
					}
					if (workTimeObjectType === 'COMBINED') {
						sumOfEngineTime += parseInt(machineWorkTimeObjects[objectIndex].OtherMachineData.EngineTime);
					}
					
				}
			
			}
		}
	}
	
	var engineTimeInHours = sumOfEngineTime / (60 * 60);
	engineTimeInHours = Math.round(engineTimeInHours * 100) / 100;
	
	return engineTimeInHours;
}

/*
 * PRODUCTION
 * Returns operator specific sum of forwarder load count AS INTEGER
 */
function getSumOfForwarderLoadCount(momAsJSON, operatorKey) {

	var machineWorkTimeObjects;
	var machineWorkTimeObject;
	var forwarderDataObjects;
	var jsonObject = JSON.parse(momAsJSON);
	var sumOfForwarderLoadCount = 0;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}
		
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (operatorKey === machineWorkTimeObjects.OperatorKey && machineWorkTimeObjects.OtherMachineData.ForwarderData) {
			forwarderDataObjects = machineWorkTimeObjects.OtherMachineData.ForwarderData;
			if (getJsonReturnType(forwarderDataObjects) === 'Object') {
				sumOfForwarderLoadCount = parseInt(forwarderDataObjects.NumberOfLoads);
			}
			if (getJsonReturnType(forwarderDataObjects) === 'Array') {
				for (var objIndex in forwarderDataObjects) {
					sumOfForwarderLoadCount += parseInt(forwarderDataObjects[objIndex].NumberOfLoads);
				}
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (var objectIndex in machineWorkTimeObjects) {
			machineWorkTimeObject = machineWorkTimeObjects[objectIndex];
			if (operatorKey === machineWorkTimeObjects[objectIndex].OperatorKey && machineWorkTimeObjects[objectIndex].OtherMachineData.ForwarderData) {
				forwarderDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData.ForwarderData;
				if (getJsonReturnType(forwarderDataObjects) === 'Object') {
					sumOfForwarderLoadCount += parseInt(forwarderDataObjects.NumberOfLoads);
				}
				if (getJsonReturnType(forwarderDataObjects) === 'Array') {
					for (var objIndex in forwarderDataObjects) {
						sumOfForwarderLoadCount += parseInt(forwarderDataObjects[objIndex].NumberOfLoads);
					}
				}
			}
		}
	}
	
	//console.log('Sum of forwarder load count: ' + sumOfForwarderLoadCount);
	
	return sumOfForwarderLoadCount;

}

/*
 * PRODUCTION
 * Returns sum of operator work time (seconds) in requested category AS INTEGER
 * Request category strings: MachineWorkTime
 *							 Meal break
 *						     Planning outside machine
 *						     Travelling to work outside machine
 *						     Other work outside machine
 */
function getOperatorWorkTimeInCategory(momAsJSON, categoryString, operatorKey) { 

	var jsonObject = JSON.parse(momAsJSON);
	var sumOfWorkTimeInCategory = 0;
	var operatorWorkTimeObjects;
	
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.OperatorWorkTime) {
		
		operatorWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.OperatorWorkTime;
		
		if (getJsonReturnType(operatorWorkTimeObjects) === 'Object') {
			//console.log('Operator work time object is an OBJECT');
			if (operatorKey === operatorWorkTimeObjects.OperatorKey && operatorWorkTimeObjects.$.OperatorWorkTimeCategory === categoryString) {
				//console.log('Operator work time category in data: ' + operatorWorkTimeObjects.$.OperatorWorkTimeCategory);
				//console.log('Operator work time category in request: ' + categoryString);
				sumOfWorkTimeInCategory = parseInt(operatorWorkTimeObjects.MonitoringTimeLength);
			}
		}
		if (getJsonReturnType(operatorWorkTimeObjects) === 'Array') {
			//console.log('Operator work time object is an ARRAY');
			for (var objectIndex in operatorWorkTimeObjects) {
				if (operatorKey === operatorWorkTimeObjects[objectIndex].OperatorKey && operatorWorkTimeObjects[objectIndex].$.OperatorWorkTimeCategory === categoryString) {
					sumOfWorkTimeInCategory += parseInt(operatorWorkTimeObjects[objectIndex].MonitoringTimeLength);
				}
			}
		}
		
	}
	//console.log('Operator work time: Sum of ' + categoryString + ' is ' + sumOfWorkTimeInCategory);
	return sumOfWorkTimeInCategory;
	
}

/*
 * PRODUCTION
 * Returns operator specific machine work time (seconds) in requested category AS INTEGER
 * Request category string: Processing
 *                          Terrain travel
 *                          Other work
 *
 * KNOWN ISSUES: DOES NOT HANDLE 'OTHER WORK' CATEGORY AS IT SHOULD. ISSUES WITH COMBINED MODEL!
 */
function getMachineWorkTimeInCategory(momAsJSON, categoryString, operatorKey) {
	
	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var sumOfWorkTimeInCategory = 0;
	var machineWorkTimeObjectType;
	var combinedMachineRunTimeObjects;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
		//console.log('Work time object type is INDIVIDUAL');
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
		//console.log('Work time object type is COMBINED');
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		//console.log('Work time object return type is OBJECT');
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			//console.log('Operator key match - is ' + operatorKey);
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				//console.log('Processing individual');
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'String') {
					//console.log('Individual machine run time category json object is STRING');
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory === categoryString) {
						//console.log('Category string match - is ' + categoryString);
						sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
					}
				}
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'Object') {
					//console.log('Individual machine run time category json object is OBJECT'); 
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory['_'] === categoryString) {
						//console.log('Category string match - is ' + categoryString);
						sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
					}
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') {
				//console.log('Processing combined');
				combinedMachineRunTimeObjects = machineWorkTimeObjects.CombinedMachineRunTime;
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
					//console.log('Combinined machine run time object type is OBJECT');
					if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
						//console.log('Category string match - is ' + categoryString);
						sumOfWorkTimeInCategory = parseInt(combinedMachineRunTimeObjects.TimeLength);
					}
				}
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
					//console.log('Combinined machine run time object type is ARRAY');
					for (var objIndex in combinedMachineRunTimeObjects) {
						if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
						}
					}
				}
			}
		}
	
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		//console.log('Work time object return type is ARRAY');
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				//console.log('Operator key match - is ' + operatorKey);
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					//console.log('Processing individual');
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'String') {
						//console.log('Individual machine run time category json object is STRING');
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
						}
					}
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'Object') {
						//console.log('Individual machine run time category json object is OBJECT'); 
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory['_'] === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
						} 
					}
				}
				if (machineWorkTimeObjectType === 'COMBINED') {
					//console.log('Processing combined');
					combinedMachineRunTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineRunTime;
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
						//console.log('Combinined machine run time object type is OBJECT');
						if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects.TimeLength);
						}
					}
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
						//console.log('Combinined machine run time object type is ARRAY');
						for (var objIndex in combinedMachineRunTimeObjects) {
							if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
								//console.log('Category string match - is ' + categoryString);
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
							}
						}
					}
				}
			}
		}
	}

	//console.log('SUM of operator ' + operatorKey + ' machine work time in category ' + categoryString + ' is ' + sumOfWorkTimeInCategory);
	
	return sumOfWorkTimeInCategory;
	
}


/*
 * DEVELOPMENT PRODUCTION
 * Returns operator specific machine work time in requested machine work time sub category AS STRING
 * Request category string: Processing
 *                          Terrain travel
 *                          Other work
 * Request sub category string: Processing categories
 *                              - SingleTreeProcessing
 *		                        - MultiTreeProcessing
 *  	                        - SingleTreeFelling
 *		                        - MultiTreeFelling
 *  							Other work
 *	                            - Road travel
 *                              - Preparing strip roads
 *                              - Towing other machine
 *                              - Roadside loading of truck
 *                              - Unspecified
 *
 * DISCLAIMER: RETURNS CURRENTLY TIME FOR ONLY ROAD TRAVEL
 */
function getMachineWorkTimeInSubCategory(momAsJSON, categoryString, subcategoryString, operatorKey) {
	
	//console.log(categoryString);
	
	var jsonObject = JSON.parse(momAsJSON);
	
	if (categoryString === 'Terrain travel') {
		return 0;
	}
	
	var machineWorkTimeObjects;
	var sumOfWorkTimeInCategory = 0;
	var machineWorkTimeObjectType;
	var combinedMachineRunTimeObjects;
	
	var SUM_OF_ROAD_TRAVEL = 0;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
		//console.log('Work time object type is INDIVIDUAL');
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
		//console.log('Work time object type is COMBINED');
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'String') {
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory === categoryString) {
						sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
						if (categoryString === 'Other work') {
							//console.log('OTHER WORK CATEGORY: ' + machineWorkTimeObjects.IndividualMachineRunTimeCategory);
						}
					}
				}
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'Object') {
					//console.log('Individual machine run time category json object is OBJECT'); 
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory['_'] === categoryString) {
						sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
						if (categoryString === 'Other work') {
							if (machineWorkTimeObjects.IndividualMachineRunTimeCategory.$.otherWorkCategory === 'Road travel') {
								//console.log('OTHER WORK CATEGORY: ' + machineWorkTimeObjects.IndividualMachineRunTimeCategory.$.otherWorkCategory);
								SUM_OF_ROAD_TRAVEL += parseInt(machineWorkTimeObjects.MonitoringTimeLength);
							}
						}
					}
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') {
				combinedMachineRunTimeObjects = machineWorkTimeObjects.CombinedMachineRunTime;
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
					if (getJsonReturnType(combinedMachineRunTimeObjects.MachineRunTimeCategory) === 'Object') {
						if (combinedMachineRunTimeObjects.MachineRunTimeCategory['_'] === categoryString) {
							sumOfWorkTimeInCategory = parseInt(combinedMachineRunTimeObjects.TimeLength);
							if (categoryString === 'Other work') {
								if (combinedMachineRunTimeObjects.MachineRunTimeCategory.$.otherWorkCategory === 'Road travel') {
									//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects.MachineRunTimeCategory.$.otherWorkCategory);
									SUM_OF_ROAD_TRAVEL += parseInt(combinedMachineRunTimeObjects.TimeLength);
								}
							}
						}
					} else {
						if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
							sumOfWorkTimeInCategory = parseInt(combinedMachineRunTimeObjects.TimeLength);
							if (categoryString === 'Other work') {
								//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects.MachineRunTimeCategory);
							}
						}
					}
				}
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
					for (var objIndex in combinedMachineRunTimeObjects) {
						//console.log('run time category object is ' + getJsonReturnType(combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory));
						if (getJsonReturnType(combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory) === 'Object') {
							if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory['_'] === categoryString) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
								if (categoryString === 'Other work') {
									if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory.$.otherWorkCategory === 'Road travel') {
										//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory.$.otherWorkCategory);
										SUM_OF_ROAD_TRAVEL += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
									}
								}
							}
						} else {
							if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
								if (categoryString === 'Other work') {
									//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory);
								}
							}
						}
					}
				}
			}
		}
	
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'String') {
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === categoryString) {
							sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
							if (categoryString === 'Other work') {
								//console.log('OTHER WORK CATEGORY: ' + machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory);
							}
						}
					}
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'Object') {
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory['_'] === categoryString) {
							sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
							if (categoryString === 'Other work') {
								if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory.$.otherWorkCategory === 'Road travel') {
									//console.log('OTHER WORK CATEGORY: ' + machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory.$.otherWorkCategory);
									SUM_OF_ROAD_TRAVEL += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
								}
							}
						} 
					}
				}
				if (machineWorkTimeObjectType === 'COMBINED') {
					combinedMachineRunTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineRunTime;
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
						if (getJsonReturnType(combinedMachineRunTimeObjects.MachineRunTimeCategory) === 'Object') {
							if (combinedMachineRunTimeObjects.MachineRunTimeCategory['_'] === categoryString) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects.TimeLength);
								if (categoryString === 'Other work') {
									if (combinedMachineRunTimeObjects.MachineRunTimeCategory['_'].otherWorkCategory === 'Road travel') {
										//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects.MachineRunTimeCategory['_'].otherWorkCategory);
										SUM_OF_ROAD_TRAVEL += parseInt(combinedMachineRunTimeObjects.TimeLength);
									}
								}
							}
						} else {
							if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects.TimeLength);
								if (categoryString === 'Other work') {
									//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects.MachineRunTimeCategory);
								}
							}
						}
					}
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
						for (var objIndex in combinedMachineRunTimeObjects) {
							if (getJsonReturnType(combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory) === 'Object') {
								if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory['_'] === categoryString) {
									sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
									if (categoryString === 'Other work') {
										if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory['_'].otherWorkCategory === 'Road travel') {
											//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory['_'].otherWorkCategory);
											SUM_OF_ROAD_TRAVEL += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
										}
									}
								}
							} else {
								if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
									sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
									if (categoryString === 'Other work') {
										//console.log('OTHER WORK CATEGORY: ' + combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	//console.log('SUM of operator ' + operatorKey + ' machine work time in category ' + categoryString + ' is ' + sumOfWorkTimeInCategory);
	
	//return sumOfWorkTimeInCategory;
	//console.log('SUM OF ROAD TRAVEL: ' + SUM_OF_ROAD_TRAVEL);
	return SUM_OF_ROAD_TRAVEL;

}

/*
 * PRODUCTION
 * Returns filtered operator specific machine work time (seconds) in requested category AS INTEGER
 * Request category string: Processing
 *                          Terrain travel
 *                          Other work
 */
function getFilteredMachineWorkTimeInCategory(momAsJSON, categoryString, operatorKey, filterValueSeconds) {
	
	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var sumOfWorkTimeInCategory = 0;
	var machineWorkTimeObjectType;
	var combinedMachineRunTimeObjects;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
		//console.log('Work time object type is INDIVIDUAL');
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
		//console.log('Work time object type is COMBINED');
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		//console.log('Work time object return type is OBJECT');
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			//console.log('Operator key match - is ' + operatorKey);
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				//console.log('Processing individual');
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'String') {
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory === categoryString) {
						if (machineWorkTimeObjects.MonitoringTimeLength < filterValueSeconds) {
							sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
						}
					}
				}
				if (getJsonReturnType(machineWorkTimeObjects.IndividualMachineRunTimeCategory) === 'Object') {
					if (machineWorkTimeObjects.IndividualMachineRunTimeCategory['_'] === categoryString) {
						if (machineWorkTimeObjects.MonitoringTimeLength < filterValueSeconds) {
							sumOfWorkTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
						}
					}
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') { 
				//console.log('Processing combined');
				combinedMachineRunTimeObjects = machineWorkTimeObjects.CombinedMachineRunTime;
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
					//console.log('Combinined machine run time object type is OBJECT');
					if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
						//console.log('Category string match - is ' + categoryString);
						if (combinedMachineRunTimeObjects.TimeLength < filterValueSeconds) {
							sumOfWorkTimeInCategory = parseInt(combinedMachineRunTimeObjects.TimeLength);
						}
					}
				}
				if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
					//console.log('Combinined machine run time object type is ARRAY');
					for (var objIndex in combinedMachineRunTimeObjects) {
						if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							if (combinedMachineRunTimeObjects[objIndex].TimeLength < filterValueSeconds) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
							}
						}
					}
				}
			}
		}
	
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		//console.log('Work time object return type is ARRAY');
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				//console.log('Operator key match - is ' + operatorKey);
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					//console.log('Processing individual');
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'String') {
						//console.log('Individual machine run time category json object is STRING');
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							if (machineWorkTimeObjects[objectIndex].MonitoringTimeLength < filterValueSeconds) {
								sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
							}
						}
					}
					if (getJsonReturnType(machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) === 'Object') {
						//console.log('Individual machine run time category json object is OBJECT'); 
						if (machineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory['_'] === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							if (machineWorkTimeObjects[objectIndex].MonitoringTimeLength < filterValueSeconds) {
								sumOfWorkTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
							}
						} 
					}
				}
				if (machineWorkTimeObjectType === 'COMBINED') {
					//console.log('Processing combined');
					combinedMachineRunTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineRunTime;
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Object') {
						//console.log('Combinined machine run time object type is OBJECT');
						if (combinedMachineRunTimeObjects.MachineRunTimeCategory === categoryString) {
							//console.log('Category string match - is ' + categoryString);
							if (combinedMachineRunTimeObjects.TimeLength < filterValueSeconds) {
								sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects.TimeLength);
							}
						}
					}
					if (getJsonReturnType(combinedMachineRunTimeObjects) === 'Array') {
						//console.log('Combinined machine run time object type is ARRAY');
						for (var objIndex in combinedMachineRunTimeObjects) {
							if (combinedMachineRunTimeObjects[objIndex].MachineRunTimeCategory === categoryString) {
								//console.log('Category string match - is ' + categoryString);
								if (combinedMachineRunTimeObjects[objIndex].TimeLength < filterValueSeconds) {
									sumOfWorkTimeInCategory += parseInt(combinedMachineRunTimeObjects[objIndex].TimeLength);
								}
							}
						}
					}
				}
			}
		}
	}

	//console.log('Filtered SUM of operator ' + operatorKey + ' machine work time in category ' + categoryString + ' is ' + sumOfWorkTimeInCategory);
	
	return sumOfWorkTimeInCategory;
	
}

/*
 * PRODUCTION
 * Returns total sum of operator specific machine down time (seconds) AS INTEGER
 */
function getMachineDownTime(momAsJSON, operatorKey) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var machineDownTimeObjects;
	var sumOfDownTimeInCategory = 0;
	var machineWorkTimeObjectType;
	var combinedMachineDownTimeObjects;
	var hasDownTimeObjects = false;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
	}

	//console.log('DOWN TIME: Work time object is individual');
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		//console.log('DOWN TIME: Work time object JSON type is object');
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			//console.log('DOWN TIME: Operator key match - is ' + operatorKey);
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				if (machineWorkTimeObjects.IndividualMachineDownTime) {
					//console.log('DOWN TIME: Has individual down time');
					sumOfDownTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') { 
				if (machineWorkTimeObjects.CombinedMachineDownTime) {
					//console.log('DOWN TIME: Has combined down time');
					combinedMachineDownTimeObjects = machineWorkTimeObjects.CombinedMachineDownTime;
					if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Object') {
						//console.log('DOWN TIME: Combined down time JSON type is object');
						sumOfDownTimeInCategory = parseInt(combinedMachineDownTimeObjects.TimeLength);
					}
					if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Array') {
						//console.log('DOWN TIME: Combined down time JSON type is array');
						for (var objIndex in combinedMachineDownTimeObjects) {
							sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects[objIndex].TimeLength);
						}
					}
				}
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		//console.log('DOWN TIME: Work time object JSON type is object');
		for (var objectIndex in machineWorkTimeObjects) {
			
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				//console.log('DOWN TIME: Operator key match - is ' + operatorKey);
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					if (machineWorkTimeObjects[objectIndex].IndividualMachineDownTime) {
						//console.log('DOWN TIME: Has individual down time');
						//console.log('DOWN TIME: Object index ' + objectIndex);
						sumOfDownTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
					}
				}
				
				if (machineWorkTimeObjectType === 'COMBINED') {
					if (machineWorkTimeObjects[objectIndex].CombinedMachineDownTime) {
						//console.log('DOWN TIME: Has combined down time');
						//combinedMachineDownTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineDownTime;
						if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Object') {
							//console.log('DOWN TIME: Combined down time JSON type is object');
							sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects.TimeLength);
						}
						if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Array') {
							//console.log('DOWN TIME: Combined down time JSON type is array');
							for (var objIndex in combinedMachineDownTimeObjects) {
								sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects[objIndex].TimeLength);
							}
						}
					}
				}

			}
		}
	}

	return sumOfDownTimeInCategory;
}

/*
 * PRODUCTION
 * Returns filtered total sum of operator specific machine down time (seconds) AS INTEGER
 */
function getFilteredMachineDownTime(momAsJSON, operatorKey, filterValueSeconds) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var machineDownTimeObjects;
	var sumOfDownTimeInCategory = 0;
	var machineWorkTimeObjectType;
	var combinedMachineDownTimeObjects;
	var hasDownTimeObjects = false;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
	}

	//console.log('DOWN TIME: Work time object is individual');
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		//console.log('DOWN TIME: Work time object JSON type is object');
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			//console.log('DOWN TIME: Operator key match - is ' + operatorKey);
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				if (machineWorkTimeObjects.IndividualMachineDownTime) {
					//console.log('DOWN TIME: Has individual down time');
					if (machineWorkTimeObjects.MonitoringTimeLength < filterValueSeconds) {
						sumOfDownTimeInCategory = parseInt(machineWorkTimeObjects.MonitoringTimeLength);
					}
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') { 
				if (machineWorkTimeObjects.CombinedMachineDownTime) {
					//console.log('DOWN TIME: Has combined down time');
					combinedMachineDownTimeObjects = machineWorkTimeObjects.CombinedMachineDownTime;
					if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Object') {
						//console.log('DOWN TIME: Combined down time JSON type is object');
						if (combinedMachineDownTimeObjects.TimeLength < filterValueSeconds) {
							sumOfDownTimeInCategory = parseInt(combinedMachineDownTimeObjects.TimeLength);
						}
					}
					if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Array') {
						//console.log('DOWN TIME: Combined down time JSON type is array');
						for (var objIndex in combinedMachineDownTimeObjects) {
							if (combinedMachineDownTimeObjects[objIndex].TimeLength < filterValueSeconds) {
								sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects[objIndex].TimeLength);
							}
						}
					}
				}
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		//console.log('DOWN TIME: Work time object JSON type is object');
		for (var objectIndex in machineWorkTimeObjects) {
			
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				//console.log('DOWN TIME: Operator key match - is ' + operatorKey);
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					if (machineWorkTimeObjects[objectIndex].IndividualMachineDownTime) {
						//console.log('DOWN TIME: Has individual down time');
						//console.log('DOWN TIME: Object index ' + objectIndex);
						if (machineWorkTimeObjects[objectIndex].MonitoringTimeLength < filterValueSeconds) {
							sumOfDownTimeInCategory += parseInt(machineWorkTimeObjects[objectIndex].MonitoringTimeLength);
						}
					}
				}
				
				if (machineWorkTimeObjectType === 'COMBINED') {
					if (machineWorkTimeObjects[objectIndex].CombinedMachineDownTime) {
						//console.log('DOWN TIME: Has combined down time');
						//combinedMachineDownTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineDownTime;
						if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Object') {
							//console.log('DOWN TIME: Combined down time JSON type is object');
							if (combinedMachineDownTimeObjects.TimeLength < filterValueSeconds) {
								sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects.TimeLength);
							}
						}
						if (getJsonReturnType(combinedMachineDownTimeObjects) === 'Array') {
							//console.log('DOWN TIME: Combined down time JSON type is array');
							for (var objIndex in combinedMachineDownTimeObjects) {
								if (combinedMachineDownTimeObjects[objIndex].TimeLength < filterValueSeconds) {
									sumOfDownTimeInCategory += parseInt(combinedMachineDownTimeObjects[objIndex].TimeLength);
								}
							}
						}
					}
				}

			}
		}
	}

	return sumOfDownTimeInCategory;
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns operator specific down times in objects AS ARRAY
 */
function getDownTimeArray(momAsJSON, operatorKey) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var machineDownTimeObjects;
	var downTimeCategory;
	var machineWorkTimeObjectType;
	var downTimeStandardCode;
	var downTimeInSeconds;
	var downTimeObject = '';
	var downTimeArray = [];
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjectType === 'INDIVIDUAL') {
				if (machineWorkTimeObjects.IndividualMachineDownTime) {
					machineDownTimeObjects = machineWorkTimeObjects.IndividualMachineDownTime;
					downTimeCategory = getMachineDownTimeType(machineDownTimeObjects, 'INDIVIDUAL', 'ROOT');
					downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects, 'INDIVIDUAL', 'STANDARD_CODE');
					downTimeInSeconds = machineWorkTimeObjects.MonitoringTimeLength;
					downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
					downTimeArray.push(downTimeObject);
				}
			}
			if (machineWorkTimeObjectType === 'COMBINED') {
				if (machineWorkTimeObjects.CombinedMachineDownTime) {
					machineDownTimeObjects = machineWorkTimeObjects.CombinedMachineDownTime;
					//console.log('Has combined machine down time');
					if (getJsonReturnType(machineDownTimeObjects) === 'Object') {
						downTimeCategory = getMachineDownTimeType(machineDownTimeObjects, 'COMBINED', 'ROOT');
						downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects, 'COMBINED', 'STANDARD_CODE');
						downTimeInSeconds = machineDownTimeObjects.TimeLength;
						downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
						downTimeArray.push(downTimeObject);
					}
					if (getJsonReturnType(machineDownTimeObjects) === 'Array') {
						for (var downIndex in machineDownTimeObjects) {
							downTimeCategory = getMachineDownTimeType(machineDownTimeObjects[downIndex], 'COMBINED', 'ROOT');
							downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects[downIndex], 'COMBINED', 'STANDARD_CODE');
							downTimeInSeconds = machineDownTimeObjects[downIndex].TimeLength;
							downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
							downTimeArray.push(downTimeObject);
						}
					}
				}
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (machineWorkTimeObjectType === 'INDIVIDUAL') {
					if (machineWorkTimeObjects[objectIndex].IndividualMachineDownTime) {
						machineDownTimeObjects = machineWorkTimeObjects[objectIndex].IndividualMachineDownTime;
						downTimeCategory = getMachineDownTimeType(machineDownTimeObjects, 'INDIVIDUAL', 'ROOT');
						downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects, 'INDIVIDUAL', 'STANDARD_CODE');
						downTimeInSeconds = machineWorkTimeObjects[objectIndex].MonitoringTimeLength;
						downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
						downTimeArray.push(downTimeObject);
					}
				}
				if (machineWorkTimeObjectType === 'COMBINED') {
					if (machineWorkTimeObjects[objectIndex].CombinedMachineDownTime) {
						machineDownTimeObjects = machineWorkTimeObjects[objectIndex].CombinedMachineDownTime;
						if (getJsonReturnType(machineDownTimeObjects) === 'Object') {
							downTimeCategory = getMachineDownTimeType(machineDownTimeObjects, 'COMBINED', 'ROOT');
							downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects, 'COMBINED', 'STANDARD_CODE');
							downTimeInSeconds = machineDownTimeObjects.TimeLength;
							downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
							downTimeArray.push(downTimeObject);
						}
						if (getJsonReturnType(machineDownTimeObjects) === 'Array') {
							for (var downIndex in machineDownTimeObjects) {
								downTimeCategory = getMachineDownTimeType(machineDownTimeObjects[downIndex], 'COMBINED', 'ROOT');
								downTimeStandardCode = getMachineDownTimeType(machineDownTimeObjects[downIndex], 'COMBINED', 'STANDARD_CODE');
								downTimeInSeconds = machineDownTimeObjects[downIndex].TimeLength;
								downTimeObject = '{"downTimeCategory":"' + downTimeCategory + '", "downTimeStandardCode":"' + downTimeStandardCode + '", "downTimeInSeconds":' + downTimeInSeconds + '}';
								downTimeArray.push(downTimeObject);
							}
						}
					}
				}
			}
		}
	}
	
	return downTimeArray;
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns machine down time type AS STRING
 * Filters:	ROOT
 *			STANDARD_CODE
 *			MANUFACTURER_CODE
 * Down time object type:	INDIVIDUAL
 *							COMBINED
 */
function getMachineDownTimeType(downTimeObject, downTimeObjectType, filter) {
	
	if (filter === 'ROOT') {
		if (downTimeObject.Repair) {
			return 'Repair';
		}
		if (downTimeObject.Maintenance) {
			return 'Maintenance';
		}
		if (downTimeObject.Disturbance) {
			return 'Disturbance';
		}
		if (downTimeObject.OtherMachineDownTimeCategory) {
			return 'Other';
		}	
	}
	
	if (filter === 'STANDARD_CODE') {
		if (downTimeObject.Repair) {
			if (downTimeObject.Repair.RepairStandardCode) {
				return downTimeObject.Repair.RepairStandardCode;
			} else if (downTimeObject.Repair.OtherRepairReason) {
				return downTimeObject.Repair.OtherRepairReason;
			} else {
				return 'Undefined';
			}
		}
		if (downTimeObject.Maintenance) {
			return downTimeObject.Maintenance.MaintenanceStandardCode;
		}
		if (downTimeObject.Disturbance) {
			return downTimeObject.Disturbance.DisturbanceStandardCode;
		}
		if (downTimeObject.OtherMachineDownTimeCategory) {
			return downTimeObject.OtherMachineDownTimeCategory.OtherMachineDownTimeStandardCode;
		}
	}
	
	if (filter === 'MANUFACTURER_CODE') {
		if (downTimeObject.Repair) {
			return downTimeObject.Repair.RepairManufacturerCode.CodeDescription;
		}
		if (downTimeObject.Maintenance) {
			return downTimeObject.Maintenance.MaintenanceManufacturerCode.CodeDescription;
		}
		if (downTimeObject.Disturbance) {
			return downTimeObject.Disturbance.DisturbanceManufacturerCode.CodeDescription;
		}
		if (downTimeObject.OtherMachineDownTimeCategory) {
			return downTimeObject.OtherMachineDownTimeCategory.OtherMachineDownTimeCategoryManufacturerCode.CodeDescription;
		}
	}
	


}
 
/*
 * TBD - DEVELOPMENT
 */

function getMachineUnutilizedTime(momAsJSON, operatorKey) {

}


/*
 * DEVELOPMENT PRODUCTION
 * Returns sum of operator specific forwarded volume (m3sob and m3sub) as ARRAY
 */
 function getSumOfForwardedVolume(momAsJSON, operatorKey) {
	
	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var otherMachineDataObjects;
	var forwarderDataObjects;
	var forwarderDataObject;
	var totalForwardedVolumeObject;
	var sumOfVolume_m3sub = 0;
	var sumOfVolume_m3sob = 0;
	var finalVolumeJson;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjects.OtherMachineData) {
				otherMachineDataObjects = machineWorkTimeObjects.OtherMachineData;
				if (otherMachineDataObjects.ForwarderData) {
					forwarderDataObjects = otherMachineDataObjects.ForwarderData;
					if (getJsonReturnType(forwarderDataObjects) === 'Object') {
						if (forwarderDataObjects.TotalForwardedVolume) {
							totalForwardedVolumeObject = forwarderDataObjects.TotalForwardedVolume;
							if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sob') {
								sumOfVolume_m3sob += parseFloat(totalForwardedVolumeObject['_']);
							}
							if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sub') {
								sumOfVolume_m3sub += parseFloat(totalForwardedVolumeObject['_']);
							}
						}
					}
					if (getJsonReturnType(forwarderDataObjects) === 'Array') {
						for (var forwarderDataIndex in forwarderDataObjects) {
							forwarderDataObject = forwarderDataObjects[forwarderDataIndex];
							if (forwarderDataObject.TotalForwardedVolume) {
								totalForwardedVolumeObject = forwarderDataObject.TotalForwardedVolume;
								if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sob') {
									sumOfVolume_m3sob += parseFloat(totalForwardedVolumeObject['_']);
								}
								if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sub') {
									sumOfVolume_m3sub += parseFloat(totalForwardedVolumeObject['_']);
								}
							}
						}
					}
				}	
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
					otherMachineDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData;
					if (otherMachineDataObjects.ForwarderData) {
						forwarderDataObjects = otherMachineDataObjects.ForwarderData;
						if (getJsonReturnType(forwarderDataObjects) === 'Object') {
							if (forwarderDataObjects.TotalForwardedVolume) {
								totalForwardedVolumeObject = forwarderDataObjects.TotalForwardedVolume;
								if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sob') {
									sumOfVolume_m3sob += parseFloat(totalForwardedVolumeObject['_']);
								}
								if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sub') {
									sumOfVolume_m3sub += parseFloat(totalForwardedVolumeObject['_']);
								}
							}
						}
						if (getJsonReturnType(forwarderDataObjects) === 'Array') {
							for (var forwarderDataIndex in forwarderDataObjects) {
								forwarderDataObject = forwarderDataObjects[forwarderDataIndex];
								if (forwarderDataObject.TotalForwardedVolume) {
									totalForwardedVolumeObject = forwarderDataObject.TotalForwardedVolume;
									if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sob') {
										sumOfVolume_m3sob += parseFloat(totalForwardedVolumeObject['_']);
									}
									if (totalForwardedVolumeObject.$.forwardedVolumeCategory === 'Volume, m3sub') {
										sumOfVolume_m3sub += parseFloat(totalForwardedVolumeObject['_']);
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	finalVolumeJson = '{"m3sob":' + sumOfVolume_m3sob + ', "m3sub":' + sumOfVolume_m3sub + '}';
	
	return finalVolumeJson;
	
 }
 
/*
 * DEVELOPMENT PRODUCTION
 * Returns sum of operator specific volume of harvested stems as key value pairs in ARRAY
 */
function getSumOfVolumeOfStems(momAsJSON, operatorKey) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var otherMachineDataObjects;
	var harvesterDataObjects;
	var totalVolumeOfHarvestedLogsObjects;
	var sumOfVolumeOfHarvestedStems = 0;
	var finalStemVolumeArray = [];
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}
	
	var speciesGroupKeys = getSpeciesGroupKeys(momAsJSON);

	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjects.OtherMachineData) {
				otherMachineDataObjects = machineWorkTimeObjects.OtherMachineData;
				if (otherMachineDataObjects.HarvesterData) {
					harvesterDataObjects = otherMachineDataObjects.HarvesterData;
					
					if (getJsonReturnType(harvesterDataObjects) === 'Object') {
						for (var speciesGroupKey in speciesGroupKeys) {
							if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
								if (harvesterDataObjects.TotalVolumeOfHarvestedLogs) {
								totalVolumeOfHarvestedLogsObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
								for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
									if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
										sumOfVolumeOfHarvestedStems = parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
									}
								}
								}
							}
							finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
							sumOfVolumeOfHarvestedStems = 0;
						}
					}
					
					if (getJsonReturnType(harvesterDataObjects) === 'Array') {
						for (var speciesGroupKey in speciesGroupKeys) {
							for (var objIndex in harvesterDataObjects) {
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
									totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].TotalVolumeOfHarvestedLogs;
									for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
										if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
											sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
										}
									}
								}
							}
							finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
							sumOfVolumeOfHarvestedStems = 0;
						}
					}
					
				}
		
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		
		for (var speciesGroupKey in speciesGroupKeys) {
			for (var objectIndex in machineWorkTimeObjects) {
				if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
					if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
						otherMachineDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData;
						if (otherMachineDataObjects.HarvesterData) {
							harvesterDataObjects = otherMachineDataObjects.HarvesterData;
							if (getJsonReturnType(harvesterDataObjects) === 'Object') {
								
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
									totalVolumeOfHarvestedLogsObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
									for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
										if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
											sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
										}
									}
								}
								
							}
							if (getJsonReturnType(harvesterDataObjects) === 'Array') {
								for (var objIndex in harvesterDataObjects) {
									if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
										if (harvesterDataObjects[objIndex].$.TotalVolumeOfHarvestedLogs) {
											totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].$.TotalVolumeOfHarvestedLogs;
										} else {
											totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].TotalVolumeOfHarvestedLogs;
										}
										for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
											if (totalVolumeOfHarvestedLogsObjects[volumeIndex].harvestedLogsVolumeCategory === 'm3sob') {
												sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
											}
											if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
												sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
											}
										}
									}
								}
								
							}
					
						}
						
		
					}
				}
		
			}
			finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
			sumOfVolumeOfHarvestedStems = 0;
		}
		
	}
	
	return finalStemVolumeArray;
	
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns sum of operator specific filtered volume of harvested stems as key value pairs in ARRAY
 */
function getSumOfVolumeOfStems_volumeFiltered(momAsJSON, operatorKey, volumeFilter) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var otherMachineDataObjects;
	var harvesterDataObjects;
	var totalVolumeOfHarvestedLogsObjects;
	var sumOfVolumeOfHarvestedStems = 0;
	var finalStemVolumeArray = [];
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}
	
	var speciesGroupKeys = getSpeciesGroupKeys(momAsJSON);

	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjects.OtherMachineData) {
				otherMachineDataObjects = machineWorkTimeObjects.OtherMachineData;
				if (otherMachineDataObjects.HarvesterData) {
					harvesterDataObjects = otherMachineDataObjects.HarvesterData;
					
					if (getJsonReturnType(harvesterDataObjects) === 'Object') {
						for (var speciesGroupKey in speciesGroupKeys) {
							if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
								if (harvesterDataObjects.TotalVolumeOfHarvestedLogs) {
								totalVolumeOfHarvestedLogsObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
								for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
									if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === volumeFilter) {
										sumOfVolumeOfHarvestedStems = parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
									}
								}
								}
							}
							finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
							sumOfVolumeOfHarvestedStems = 0;
						}
					}
					
					if (getJsonReturnType(harvesterDataObjects) === 'Array') {
						for (var speciesGroupKey in speciesGroupKeys) {
							for (var objIndex in harvesterDataObjects) {
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
									totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].TotalVolumeOfHarvestedLogs;
									for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
										if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === volumeFilter) {
											sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
										}
									}
								}
							}
							finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
							sumOfVolumeOfHarvestedStems = 0;
						}
					}
					
				}
		
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') {
		
		for (var speciesGroupKey in speciesGroupKeys) {
			for (var objectIndex in machineWorkTimeObjects) {
				if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
					if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
						otherMachineDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData;
						if (otherMachineDataObjects.HarvesterData) {
							harvesterDataObjects = otherMachineDataObjects.HarvesterData;
							if (getJsonReturnType(harvesterDataObjects) === 'Object') {
								
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
									totalVolumeOfHarvestedLogsObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
									for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
										if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === volumeFilter) {
											sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
										}
									}
								}
								
							}
							if (getJsonReturnType(harvesterDataObjects) === 'Array') {
								for (var objIndex in harvesterDataObjects) {
									if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
										if (harvesterDataObjects[objIndex].$.TotalVolumeOfHarvestedLogs) {
											totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].$.TotalVolumeOfHarvestedLogs;
										} else {
											totalVolumeOfHarvestedLogsObjects = harvesterDataObjects[objIndex].TotalVolumeOfHarvestedLogs;
										}
										for (volumeIndex in totalVolumeOfHarvestedLogsObjects) {
											if (totalVolumeOfHarvestedLogsObjects[volumeIndex].harvestedLogsVolumeCategory === volumeFilter) {
												sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
											}
											if (totalVolumeOfHarvestedLogsObjects[volumeIndex].$.harvestedLogsVolumeCategory === volumeFilter) {
												sumOfVolumeOfHarvestedStems += parseFloat(totalVolumeOfHarvestedLogsObjects[volumeIndex]['_']);
											}
										}
									}
								}
								
							}
					
						}
						
		
					}
				}
		
			}
			finalStemVolumeArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
			sumOfVolumeOfHarvestedStems = 0;
		}
		
	}
	
	return finalStemVolumeArray;
	
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns recoded (species group) sum of harvested stems filtered volume aggregate as ARRAY
 */
function getSumOfVolumeOfStemsRecoded_volumeFiltered(momAsJSON, operatorKey, volumeFilter) {

	var sumOfVolumeOfHarvestedStemsArray_volumeFiltered = getSumOfVolumeOfStems(momAsJSON, operatorKey, volumeFilter);
	var arrayItem;
	var jsonObject;
	var speciesGroupKey;
	var speciesGroupName;
	var speciesGroupNameRecoded;
	var sumOfVolumeOfHarvestedStems;
	
	var recodedStemNameArray = [];
	var recodedStemNameArrayUnique = [];
	
	var sumOfVolumeOfHarvestedStemsArrayRecoded = [];
	var sumOfVolumeOfHarvestedStemsArrayRecodedFinal = [];
	
	for (var itemIndex in sumOfVolumeOfHarvestedStemsArray_volumeFiltered) {
		arrayItem = sumOfVolumeOfHarvestedStemsArray_volumeFiltered[itemIndex];
		//console.log(arrayItem);
		jsonObject = JSON.parse(arrayItem);
		speciesGroupKey = jsonObject.speciesGroupKey;
		speciesGroupName = getSpeciesGroupName(momAsJSON, speciesGroupKey.toString());
		speciesGroupNameRecoded = getRecodedSpeciesGroupString(speciesGroupName);
		recodedStemNameArray.push(speciesGroupNameRecoded);
		sumOfVolumeOfHarvestedStems = jsonObject.sumOfVolumeOfHarvestedStems;
		sumOfVolumeOfHarvestedStemsArrayRecoded.push('{"speciesGroupName":"' + speciesGroupNameRecoded + '","sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
	}
	
	recodedStemNameArrayUnique = uniq(recodedStemNameArray);
	
	var volumeSum = 0;
	for (var stemIndex in recodedStemNameArrayUnique) {
		speciesGroupNameRecoded = recodedStemNameArrayUnique[stemIndex];
		for (var stemCountArrayIndex in sumOfVolumeOfHarvestedStemsArrayRecoded) {
			jsonObject = JSON.parse(sumOfVolumeOfHarvestedStemsArrayRecoded[stemCountArrayIndex]);
			if (jsonObject.speciesGroupName === speciesGroupNameRecoded) {
				volumeSum += jsonObject.sumOfVolumeOfHarvestedStems;
			}
		}
		sumOfVolumeOfHarvestedStemsArrayRecodedFinal.push('{"speciesGroupName":"' + recodedStemNameArrayUnique[stemIndex] + '","sumOfVolumeOfHarvestedStems":' + volumeSum + '}');
		volumeSum = 0;
	}
	
	return sumOfVolumeOfHarvestedStemsArrayRecodedFinal;
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns recoded (species group) sum of harvested stems volume aggregate as ARRAY
 */
function getSumOfVolumeOfStemsRecoded(momAsJSON, operatorKey) {

	var sumOfVolumeOfHarvestedStemsArray = getSumOfVolumeOfStems(momAsJSON, operatorKey);
	var arrayItem;
	var jsonObject;
	var speciesGroupKey;
	var speciesGroupName;
	var speciesGroupNameRecoded;
	var sumOfVolumeOfHarvestedStems;
	
	var recodedStemNameArray = [];
	var recodedStemNameArrayUnique = [];
	
	var sumOfVolumeOfHarvestedStemsArrayRecoded = [];
	var sumOfVolumeOfHarvestedStemsArrayRecodedFinal = [];
	
	for (var itemIndex in sumOfVolumeOfHarvestedStemsArray) {
		arrayItem = sumOfVolumeOfHarvestedStemsArray[itemIndex];
		//console.log(arrayItem);
		jsonObject = JSON.parse(arrayItem);
		speciesGroupKey = jsonObject.speciesGroupKey;
		speciesGroupName = getSpeciesGroupName(momAsJSON, speciesGroupKey.toString());
		speciesGroupNameRecoded = getRecodedSpeciesGroupString(speciesGroupName);
		recodedStemNameArray.push(speciesGroupNameRecoded);
		sumOfVolumeOfHarvestedStems = jsonObject.sumOfVolumeOfHarvestedStems;
		sumOfVolumeOfHarvestedStemsArrayRecoded.push('{"speciesGroupName":"' + speciesGroupNameRecoded + '","sumOfVolumeOfHarvestedStems":' + sumOfVolumeOfHarvestedStems + '}');
	}
	
	recodedStemNameArrayUnique = uniq(recodedStemNameArray);
	
	var volumeSum = 0;
	for (var stemIndex in recodedStemNameArrayUnique) {
		speciesGroupNameRecoded = recodedStemNameArrayUnique[stemIndex];
		for (var stemCountArrayIndex in sumOfVolumeOfHarvestedStemsArrayRecoded) {
			jsonObject = JSON.parse(sumOfVolumeOfHarvestedStemsArrayRecoded[stemCountArrayIndex]);
			if (jsonObject.speciesGroupName === speciesGroupNameRecoded) {
				volumeSum += jsonObject.sumOfVolumeOfHarvestedStems;
			}
		}
		sumOfVolumeOfHarvestedStemsArrayRecodedFinal.push('{"speciesGroupName":"' + recodedStemNameArrayUnique[stemIndex] + '","sumOfVolumeOfHarvestedStems":' + volumeSum + '}');
		volumeSum = 0;
	}
	
	return sumOfVolumeOfHarvestedStemsArrayRecodedFinal;
}
 
/*
 * PRODUCTION
 * Returns operator specific sum of harvester stem count as key value pairs in ARRAY
 */
function getSumOfHarvestedStems(momAsJSON, operatorKey) {
	
	var jsonObject = JSON.parse(momAsJSON);
	
	var machineWorkTimeObjects;
	var machineWorkTimeObjectType;
	var hasHarvesterData = false;
	var otherMachineDataObjects;
	var harvesterDataObjects;
	var sumOfHarvestedStems = 0;
	var finalStemCountArray = [];
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
		machineWorkTimeObjectType = 'INDIVIDUAL';
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
		machineWorkTimeObjectType = 'COMBINED';
	}
	
	var speciesGroupKeys = getSpeciesGroupKeys(momAsJSON);
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OperatorKey === operatorKey) {
			if (machineWorkTimeObjects.OtherMachineData) {
				otherMachineDataObjects = machineWorkTimeObjects.OtherMachineData;
				if (otherMachineDataObjects.HarvesterData) {
					harvesterDataObjects = otherMachineDataObjects.HarvesterData;
					
					if (getJsonReturnType(harvesterDataObjects) === 'Object') {
						for (var speciesGroupKey in speciesGroupKeys) {
							if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
								sumOfHarvestedStems = parseInt(harvesterDataObjects.NumberOfHarvestedStems);
							}
							finalStemCountArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfHarvestedStems":' + sumOfHarvestedStems + '}');
							sumOfHarvestedStems = 0;
						}
					}
					
					if (getJsonReturnType(harvesterDataObjects) === 'Array') {
						for (var speciesGroupKey in speciesGroupKeys) {
							for (var objIndex in harvesterDataObjects) {
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
									sumOfHarvestedStems += parseInt(harvesterDataObjects[objIndex].NumberOfHarvestedStems);
								}
							}
							finalStemCountArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfHarvestedStems":' + sumOfHarvestedStems + '}');
							sumOfHarvestedStems = 0;
						}
					}
					
				}
		
			}
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') { 
		for (var speciesGroupKey in speciesGroupKeys) {
			for (var objectIndex in machineWorkTimeObjects) {
				if (machineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
					if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
						otherMachineDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData;
						if (otherMachineDataObjects.HarvesterData) {
							harvesterDataObjects = otherMachineDataObjects.HarvesterData;
							if (getJsonReturnType(harvesterDataObjects) === 'Object') {
								if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects.SpeciesGroupKey) {
									sumOfHarvestedStems += parseInt(harvesterDataObjects.NumberOfHarvestedStems);
								}
								
							}
							if (getJsonReturnType(harvesterDataObjects) === 'Array') {
								for (var objIndex in harvesterDataObjects) {
									if (speciesGroupKeys[speciesGroupKey] === harvesterDataObjects[objIndex].SpeciesGroupKey) {
										sumOfHarvestedStems += parseInt(harvesterDataObjects[objIndex].NumberOfHarvestedStems);
									}
								}
								
							}
					
						}
		
					}
				}
		
			}
			finalStemCountArray.push('{"speciesGroupKey":' + speciesGroupKeys[speciesGroupKey] + ', "sumOfHarvestedStems":' + sumOfHarvestedStems + '}');
			sumOfHarvestedStems = 0;
		}
		
	}
	
	return finalStemCountArray;
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns SpeciesGroupName as STRING
 */

 function getSpeciesGroupName(momAsJSON, speciesGroupKey) {
	
	var speciesGroupName = '';
	var jsonObject = JSON.parse(momAsJSON);
	var speciesGroupDefinitionObjects;
	
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.SpeciesGroupDefinition) {
		speciesGroupDefinitionObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.SpeciesGroupDefinition;
		if (getJsonReturnType(speciesGroupDefinitionObjects) === 'Object') {
			if (speciesGroupDefinitionObjects.SpeciesGroupKey === speciesGroupKey) {
				speciesGroupName = speciesGroupDefinitionObjects.SpeciesGroupName;
				return speciesGroupName;
			}
		}
		if (getJsonReturnType(speciesGroupDefinitionObjects) === 'Array') {
			for (var objectIndex in speciesGroupDefinitionObjects) {
				if (speciesGroupDefinitionObjects[objectIndex].SpeciesGroupKey === speciesGroupKey) {
					speciesGroupName = speciesGroupDefinitionObjects[objectIndex].SpeciesGroupName;
					return speciesGroupName;
				}
			}
		}
	}
	
	return speciesGroupName;
 }

/*
 * DEVELOPMENT PRODUCTION
 * Returns recoded (English) species group string (name) as STRING
 */
function getRecodedSpeciesGroupString(speciesGroupString) {
	
	speciesGroupString = speciesGroupString.toUpperCase();
	var recodedSpeciesGroupString;
	
	switch (speciesGroupString) {
		case 'MÄNTY':
			recodedSpeciesGroupString = 'Pine';
			break;
		case 'TALL':
			recodedSpeciesGroupString = 'Pine';
			break;
		case 'PINE':
			recodedSpeciesGroupString = 'Pine'; 
			break;
		case 'KUUSI':
			recodedSpeciesGroupString = 'Spruce';
			break;
		case 'GRAN':
			recodedSpeciesGroupString = 'Spruce';
			break;
		case 'SPRUCE':
			recodedSpeciesGroupString = 'Spruce';
			break;
		case 'KOIVU':
			recodedSpeciesGroupString = 'Birch';
			break;
		case 'BJÖRK':
			recodedSpeciesGroupString = 'Birch';
			break;
		case 'BIRCH':
			recodedSpeciesGroupString = 'Birch';
			break;
		case 'MUU PUULAJI':
			recodedSpeciesGroupString = 'Other';
		default:
			recodedSpeciesGroupString = 'Other';
	}

	return recodedSpeciesGroupString;
	
}

/*
 * DEVELOPMENT PRODUCTION
 * Returns recoded (species group) sum of harvested stems aggregate as ARRAY
 */
function getSumOfHarvestedStemsRecoded(momAsJSON, operatorKey) {

	var sumOfHarvestedStemsArray = getSumOfHarvestedStems(momAsJSON, operatorKey);
	var arrayItem;
	var jsonObject;
	var speciesGroupKey;
	var speciesGroupName;
	var speciesGroupNameRecoded;
	var sumOfHarvestedStems;
	
	var recodedStemNameArray = [];
	var recodedStemNameArrayUnique = [];
	
	var sumOfHarvestedStemsArrayRecoded = [];
	var sumOfHarvestedStemsArrayRecodedFinal = [];
	
	for (var itemIndex in sumOfHarvestedStemsArray) {
		arrayItem = sumOfHarvestedStemsArray[itemIndex];
		//console.log(arrayItem);
		jsonObject = JSON.parse(arrayItem);
		speciesGroupKey = jsonObject.speciesGroupKey;
		speciesGroupName = getSpeciesGroupName(momAsJSON, speciesGroupKey.toString());
		speciesGroupNameRecoded = getRecodedSpeciesGroupString(speciesGroupName);
		recodedStemNameArray.push(speciesGroupNameRecoded);
		sumOfHarvestedStems = jsonObject.sumOfHarvestedStems;
		sumOfHarvestedStemsArrayRecoded.push('{"speciesGroupName":"' + speciesGroupNameRecoded + '","numberOfHarvestedStems":' + sumOfHarvestedStems + '}');
	}
	
	recodedStemNameArrayUnique = uniq(recodedStemNameArray);
	
	var stemCount = 0;
	for (var stemIndex in recodedStemNameArrayUnique) {
		speciesGroupNameRecoded = recodedStemNameArrayUnique[stemIndex];
		for (var stemCountArrayIndex in sumOfHarvestedStemsArrayRecoded) {
			jsonObject = JSON.parse(sumOfHarvestedStemsArrayRecoded[stemCountArrayIndex]);
			if (jsonObject.speciesGroupName === speciesGroupNameRecoded) {
				stemCount += jsonObject.numberOfHarvestedStems;
			}
		}
		sumOfHarvestedStemsArrayRecodedFinal.push('{"speciesGroupName":"' + recodedStemNameArrayUnique[stemIndex] + '","numberOfHarvestedStems":' + stemCount + '}');
		stemCount = 0;
	}
	
	return sumOfHarvestedStemsArrayRecodedFinal;
}
 
 
/*
 * PRODUCTION
 * Returns unique species group keys as ARRAY
 */

function getSpeciesGroupKeys(momAsJSON) {

	var jsonObject = JSON.parse(momAsJSON);
	
	var speciesGroupKeys = []; 
	var machineWorkTimeObjects;
	var otherMachineDataObjects;
	var harvesterDataObjects;
	
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'INDIVIDUAL') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	}
	if (OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON) === 'COMBINED') {
		machineWorkTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	}

	if (getJsonReturnType(machineWorkTimeObjects) === 'Object') {
		if (machineWorkTimeObjects.OtherMachineData) {
			otherMachineDataObjects = machineWorkTimeObjects.OtherMachineData;
			if (otherMachineDataObjects.HarvesterData) {
				harvesterDataObjects = otherMachineDataObjects.HarvesterData;
				if (getJsonReturnType(harvesterDataObjects) === 'Object') {
					speciesGroupKeys.push(harvesterDataObjects.SpeciesGroupKey);
				}
				if (getJsonReturnType(harvesterDataObjects) === 'Array') {
					for (var objIndex in harvesterDataObjects) {
						speciesGroupKeys.push(harvesterDataObjects[objIndex].SpeciesGroupKey);
					}
				}
			}
		
		}
	}
	
	if (getJsonReturnType(machineWorkTimeObjects) === 'Array') { 
		for (var objectIndex in machineWorkTimeObjects) {
			if (machineWorkTimeObjects[objectIndex].OtherMachineData) {
				otherMachineDataObjects = machineWorkTimeObjects[objectIndex].OtherMachineData;
				if (otherMachineDataObjects.HarvesterData) {
					harvesterDataObjects = otherMachineDataObjects.HarvesterData;
					if (getJsonReturnType(harvesterDataObjects) === 'Object') {
						speciesGroupKeys.push(harvesterDataObjects.SpeciesGroupKey);
					}
					if (getJsonReturnType(harvesterDataObjects) === 'Array') {
						for (var objIndex in harvesterDataObjects) {
							speciesGroupKeys.push(harvesterDataObjects[objIndex].SpeciesGroupKey);
						}
					}
				}
		
			}
		}
	}
	
	var uniqueSpeciesGroupKeys = uniq(speciesGroupKeys);
	
	return uniqueSpeciesGroupKeys;
	
}

/*
 *
 *
 *
 * PROTOTYPE FUNCTIONS
 * NOT IN PRODUCTION
 *
 *
 *
 */




function OPERATIONAL_MONITORING_MACHINE_getIndividualMachineWorkTimeObjects(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	return jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
}

function OPERATIONAL_MONITORING_MACHINE_getCombinedMachineWorkTimeObjects(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	return jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
}

function getIndividualMachineWorkTime_operatorKey(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.OperatorKey;
}

function getCombinedMachineWorkTime_operatorKey(combinedMachineWorkTimeObject) {
	return combinedMachineWorkTimeObject.OperatorKey;
}

function getIndividualMachineWorkTime_individualMachineDownTimeObject(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.IndividualMachineDownTime;
}

/* STANFORD2010 POSSIBLE MACHINE DOWN TIME TYPES
 * Repair -> REPAIR
 * Maintenance -> MAINTENANCE
 * Disturbance -> DISTURBANCE
 * OtherMachineDownTimeCategory -> OTHER
 */
function getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(individualMachineDownTimeObject) {
	if (individualMachineDownTimeObject.Disturbance) {
		return 'DISTURBANCE';
	} else if (individualMachineDownTimeObject.Maintenance) {
		return 'MAINTENANCE';
	} else if (individualMachineDownTimeObject.Repair) {
		return 'REPAIR';
	} else if (individualMachineDownTimeObject.OtherMachineDownTimeCategory) {
		return 'OTHER';
	} else {
		return '';
	}
}

function getIndividualMachineWorkTime_individualMachineDownTime_downTimeStandardCode(individualMachineDownTimeObject) {
	var otherCode = '';
	switch (getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(individualMachineDownTimeObject)) {
		case 'DISTURBANCE':
			return individualMachineDownTimeObject.Disturbance.DisturbanceStandardCode;
			break;
		case 'MAINTENANCE':
			return individualMachineDownTimeObject.Maintenance.MaintenanceStandardCode;
			break;
		case 'REPAIR':
			return individualMachineDownTimeObject.Repair.RepairStandardCode;
			break;
		case 'OTHER':
			//console.log(individualMachineDownTimeObject);
			//if (individualMachineDownTimeObject.Other.OtherStandardCode) {
			//	otherCode =  individualMachineDownTimeObject.Other.OtherStandardCode;
			//}
			//if (individualMachineDownTimeObject.OtherMachineDownTimeCategory.OtherMachineDownTimeStandardCode) {
			return individualMachineDownTimeObject.OtherMachineDownTimeCategory.OtherMachineDownTimeStandardCode;
			//}
			//return otherCode;
			break;
		default:
			return '';
	}
}

function getIndividualMachineWorkTime_hasDownTime(individualMachineWorkTimeObject) {
	
	if (individualMachineWorkTimeObject.IndividualMachineDownTime) {
		return true;
	} else {
		return false;
	}
}

function getIndividualMachineWorkTime_getUniqueMachineRunTimeDownTimeCategories(individualMachineWorkTimeObjects, operatorKey) {

	var machineDownTimeCategories = [];
	var uniqueMachineDownTimeCategories = [];
	
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			if (getIndividualMachineWorkTime_hasDownTime(individualMachineWorkTimeObjects)) {
				console.log('object: has down time');
				machineDownTimeCategories.push(getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(getIndividualMachineWorkTime_individualMachineDownTimeObject(individualMachineWorkTimeObjects)));
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (getIndividualMachineWorkTime_hasDownTime(individualMachineWorkTimeObjects[objectIndex])) {
					console.log('array: has down time');
					machineDownTimeCategories.push(getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(getIndividualMachineWorkTime_individualMachineDownTimeObject(individualMachineWorkTimeObjects[objectIndex])));
				}
			}
		}
	}
	
	console.log('UTILITY 133: ' + machineDownTimeCategories);
	
	uniqueMachineDownTimeCategories = uniq(machineDownTimeCategories);
	
	return uniqueMachineDownTimeCategories;

}

function getIndividualMachineWorkTime_downTimeSumPerRequestedDownTimeCategory(individualMachineWorkTimeObjects, downTimeCategory, operatorKey) {

	var downTimeSum = 0;
	
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			if (getIndividualMachineWorkTime_hasDownTime(individualMachineWorkTimeObjects)) {
				if (downTimeCategory === getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(getIndividualMachineWorkTime_individualMachineDownTimeObject(individualMachineWorkTimeObjects))) {
					downTimeSum = parseInt(getIndividualMachineWorkTime_MonitoringTimeLength(individualMachineWorkTimeObjects));
				}
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (getIndividualMachineWorkTime_hasDownTime(individualMachineWorkTimeObjects[objectIndex])) {
					if (downTimeCategory === getIndividualMachineWorkTime_individualMachineDownTime_downTimeCategory(getIndividualMachineWorkTime_individualMachineDownTimeObject(individualMachineWorkTimeObjects[objectIndex]))) {
						downTimeSum += parseInt(getIndividualMachineWorkTime_MonitoringTimeLength(individualMachineWorkTimeObjects[objectIndex]));
					}
				}
			}
		}
	}
	
	return downTimeSum;

}



function getIndividualMachineWorkTime_MonitoringStartTime(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.MonitoringStartTime;
}

function getCombinedMachineWorkTime_StartTime(combinedMachineWorkTimeObject) {
	return combinedMachineWorkTimeObject.StartTime;
}

function getCombinedMachineWorkTime_CombinedEndTime(combinedMachineWorkTimeObject) {
	return combinedMachineWorkTimeObject.CombinedEndTime;
}

function getCombinedMachineWorkTime_CalculatedMonitoringTimeLength(combinedMachineWorkTimeObject) {
	return 0;
}

function getIndividualMachineWorkTime_MonitoringTimeLength(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.MonitoringTimeLength;
}

function getIndividualMachineWorkTime_IndividualMachineRunTimeCategory(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.IndividualMachineRunTimeCategory;
}

function getIndividualMachineWorkTime_IndividualUnutilizedTimeCategory(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.IndividualUnutilizedTimeCategory;
}

function getIndividualMachineWorkTime_sumOfRequestedIndividualUnutilizedTimeCategory(individualMachineWorkTimeObjects, unutilizedTimeCategory, operatorKey) {
	
	var unutilizedTimeSum = 0;
	
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			if (individualMachineWorkTimeObjects.IndividualUnutilizedTimeCategory) {
				if (unutilizedTimeCategory === individualMachineWorkTimeObjects.IndividualUnutilizedTimeCategory) {
					unutilizedTimeSum = parseInt(getIndividualMachineWorkTime_MonitoringTimeLength(individualMachineWorkTimeObjects));
				}
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (individualMachineWorkTimeObjects[objectIndex].IndividualUnutilizedTimeCategory) {
					if (unutilizedTimeCategory === individualMachineWorkTimeObjects[objectIndex].IndividualUnutilizedTimeCategory) {
						unutilizedTimeSum += parseInt(getIndividualMachineWorkTime_MonitoringTimeLength(individualMachineWorkTimeObjects[objectIndex]));
					}
				}
			}
		}
	}
	
	return unutilizedTimeSum;

}

function getIndividualMachineWorkTime_uniqueIndividualUnutilizedTimeCategories(individualMachineWorkTimeObjects, operatorKey) {
	
	var unutilizedTimeCategories = [];
	var uniqueUnutilizedTimeCategories = [];
	
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			if (individualMachineWorkTimeObjects.IndividualUnutilizedTimeCategory) {
				unutilizedTimeCategories.push(individualMachineWorkTimeObjects.IndividualUnutilizedTimeCategory);
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (individualMachineWorkTimeObjects[objectIndex].IndividualUnutilizedTimeCategory) {
					unutilizedTimeCategories.push(individualMachineWorkTimeObjects[objectIndex].IndividualUnutilizedTimeCategory);
				}
			}
		}
	}
	
	uniqueUnutilizedTimeCategories = uniq(unutilizedTimeCategories);
	
	return uniqueUnutilizedTimeCategories;
	
}

function getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObject) {
	return individualMachineWorkTimeObject.OtherMachineData;
}

function getIndividualMachineWorkTime_otherMachineData_engineTime(otherMachineDataObject) {
	return otherMachineDataObject.EngineTime;
}

function getIndividualMachineWorkTime_otherMachineData_drivenDistance(otherMachineDataObject) {
	return otherMachineDataObject.DrivenDistance;
}

function getIndividualMachineWorkTime_otherMachineData_fuelConsumption(otherMachineDataObject) {
	return otherMachineDataObject.FuelConsumption;
}

//This function is a non-optimized prototype
function getIndividualMachineWorkTime_harvesterDataAggregateArray(individualMachineWorkTimeObjects, operatorKey) {

	var otherMachineDataObjects;
	var harvesterDataObjects;
	
	var processingCategories = [];
	var uniqueProcessingCategories = [];
	
	var speciesGroupKeys = [];
	var uniqueSpeciesGroupKeys = [];
	
	//PROCESSING CATEGORIES
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects);
			harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
			if (getJsonReturnType(harvesterDataObjects) === 'Object') {
				processingCategories.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_processingCategory(harvesterDataObjects));
			} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
				for (var hdObjectIndex in harvesterDataObjects) {
					processingCategories.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_processingCategory(harvesterDataObjects[hdObjectIndex]));
				}
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects[objectIndex]);
				harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
				if (getJsonReturnType(harvesterDataObjects) === 'Object') {
					processingCategories.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_processingCategory(harvesterDataObjects));
				} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
					for (var hdObjectIndex in harvesterDataObjects) {
						processingCategories.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_processingCategory(harvesterDataObjects[hdObjectIndex]));
					}
				}
			}
		}
	}
	
	uniqueProcessingCategories = uniq(processingCategories);
	console.log(uniqueProcessingCategories); 
	
	//SPECIES GROUP KEYS
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects);
			harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
			if (getJsonReturnType(harvesterDataObjects) === 'Object') {
				speciesGroupKeys.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects));
			} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
				for (var hdObjectIndex in harvesterDataObjects) {
					speciesGroupKeys.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex]));
				}
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects[objectIndex]);
				harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
				if (getJsonReturnType(harvesterDataObjects) === 'Object') {
					speciesGroupKeys.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects));
				} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
					for (var hdObjectIndex in harvesterDataObjects) {
						speciesGroupKeys.push(getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex]));
					}
				}
			}
		}
	}
	
	uniqueSpeciesGroupKeys = uniq(speciesGroupKeys);
	
	
	//AGGREGATES
	
	var aggtegateArray = [];
	var aggregateArrayItem = [];
	var aggregateJson = '';
	var aggregateJsonItem = '';
	var numberOfHarvestedStems = 0;
	var volumeObjects;
	var volume_m3sub = 0.00;
	var volume_m3sob = 0.00;
	var monitoringTimeLengthSum = 0;
	
	aggregateJson = '{ "aggregates": [';
	
	for (var pcIndex in uniqueProcessingCategories) {
		//console.log('HARVESTER DATA: Iterating processing categories');
		//aggregateArrayItem.push(uniqueProcessingCategories[pcIndex]);
		aggregateJsonItem = '{ "processingGategoryString":"' + uniqueProcessingCategories[pcIndex] + '", "speciesGroupAggregate": [';
		//console.log('HARVESTER DATA: Processing categories array length ' + uniqueProcessingCategories.length);
		//console.log('HARVESTER DATA: Iterating processing category ' + uniqueProcessingCategories[pcIndex]);
		for (var index in uniqueSpeciesGroupKeys) {
			//console.log('HARVESTER DATA: Iterating species group keys');
			//console.log('HARVESTER DATA: Species group array length ' + uniqueSpeciesGroupKeys.length);
			//console.log('HARVESTER DATA: Iterating species group key ' + uniqueSpeciesGroupKeys[index]);
			if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
				//console.log('HARVESTER DATA: Individual machine work time object is a single OBJECT');
				//console.log('HARVESTER DATA: Operator key test. Expecting ' + operatorKey + ', is ' + individualMachineWorkTimeObjects.OperatorKey);
				if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
					//console.log('HARVESTER DATA: Operator key match');
					otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects);
					harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
					if (getJsonReturnType(harvesterDataObjects) === 'Object') {
						//console.log('HARVESTER DATA: Harvester data object is a single OBJECT');
						//console.log('HARVESTER DATA: Processing category test. Expecting ' + uniqueProcessingCategories[pcIndex] + ', is ' + harvesterDataObjects.ProcessingCategory);
						//console.log('HARVESTER DATA: Species group key test. Expecting ' + uniqueSpeciesGroupKeys[index] + ', is ' + getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects));
						if (uniqueProcessingCategories[pcIndex] === harvesterDataObjects.ProcessingCategory && uniqueSpeciesGroupKeys[index] === getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects)) {
							//console.log('HARVESTER DATA: Processing category match AND species group key match');
							//console.log('HARVESTER DATA: Getting number of harvested stems for calculation. Got ' + harvesterDataObjects.NumberOfHarvestedStems);
							numberOfHarvestedStems += parseInt(harvesterDataObjects.NumberOfHarvestedStems);
							volumeObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
							if (getJsonReturnType(volumeObjects) === 'Object') {
								if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sub') {
									volume_m3sub += parseFloat(volumeObjects['_']);
								} else if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sob') {
									volume_m3sob += parseFloat(volumeObjects['_']);
								}
							} else if (getJsonReturnType(volumeObjects) === 'Array') {
								for (var volumeIndex in volumeObjects) {
									if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sub') {
										volume_m3sub += parseFloat(volumeObjects[volumeIndex]['_']);
									} else if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
										volume_m3sob += parseFloat(volumeObjects[volumeIndex]['_']);
									}
								}
							}
							//console.log(harvesterDataObjects.NumberOfHarvestedStems);
							
						}
					} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
						//console.log('HARVESTER DATA: Harvester data object is an ARRAY');
						//console.log('HARVESTER DATA: Harvester data arrray length ' + harvesterDataObjects.length);
						for (var hdObjectIndex in harvesterDataObjects) {
							//console.log('HARVESTER DATA: Iterating harvester data object ' + hdObjectIndex);
							//console.log('HARVESTER DATA: Processing category test. Expecting ' + uniqueProcessingCategories[pcIndex] + ', is ' + harvesterDataObjects[hdObjectIndex].ProcessingCategory);
							//console.log('HARVESTER DATA: Species group key test. Expecting ' + uniqueSpeciesGroupKeys[index] + ', is ' + getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex]));
							if (uniqueProcessingCategories[pcIndex] === harvesterDataObjects[hdObjectIndex].ProcessingCategory && uniqueSpeciesGroupKeys[index] === getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex])) {
								//console.log('HARVESTER DATA: Processing category match AND species group key match');
								//console.log('HARVESTER DATA: Getting number of harvested stems for calculation. Got ' + harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
								numberOfHarvestedStems += parseInt(harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
								volumeObjects = harvesterDataObjects[hdObjectIndex].TotalVolumeOfHarvestedLogs;
								if (getJsonReturnType(volumeObjects) === 'Object') {
									if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sub') {
										volume_m3sub += parseFloat(volumeObjects['_']);
									} else if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sob') {
										volume_m3sob += parseFloat(volumeObjects['_']);
									}
								} else if (getJsonReturnType(volumeObjects) === 'Array') {
									for (var volumeIndex in volumeObjects) {
										if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sub') {
											volume_m3sub += parseFloat(volumeObjects[volumeIndex]['_']);
										} else if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
											volume_m3sob += parseFloat(volumeObjects[volumeIndex]['_']);
										}
									}
								}
								//console.log(harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
							}
						}
					}
				}
			} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
				//console.log('HARVESTER DATA: Individual machine work time object is an ARRAY');
				//console.log('HARVESTER DATA: Work time object array length ' + uniqueProcessingCategories.length);
				for (var objectIndex in individualMachineWorkTimeObjects) {
					//console.log('HARVESTER DATA: Iterating work time object ' + objectIndex);
					if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
						otherMachineDataObjects = getIndividualMachineWorkTime_otherMachineDataObjects(individualMachineWorkTimeObjects[objectIndex]);
						harvesterDataObjects = getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObjects);
						if (getJsonReturnType(harvesterDataObjects) === 'Object') {
							//console.log('HARVESTER DATA: Harvester data object is a single OBJECT');
							//console.log('HARVESTER DATA: Processing category test. Expecting ' + uniqueProcessingCategories[pcIndex] + ', is ' + harvesterDataObjects.ProcessingCategory);
							//console.log('HARVESTER DATA: Species group key test. Expecting ' + uniqueSpeciesGroupKeys[index] + ', is ' + getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects));
							if (uniqueProcessingCategories[pcIndex] === harvesterDataObjects.ProcessingCategory && uniqueSpeciesGroupKeys[index] === getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects)) {
								//console.log('HARVESTER DATA: Processing category match AND species group key match');
								//console.log('HARVESTER DATA: Getting number of harvested stems for calculation. Got ' + harvesterDataObjects.NumberOfHarvestedStems);
								numberOfHarvestedStems += parseInt(harvesterDataObjects.NumberOfHarvestedStems);
								volumeObjects = harvesterDataObjects.TotalVolumeOfHarvestedLogs;
								if (getJsonReturnType(volumeObjects) === 'Object') {
									if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sub') {
										volume_m3sub += parseFloat(volumeObjects['_']);
									} else if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sob') {
										volume_m3sob += parseFloat(volumeObjects['_']);
									}
								} else if (getJsonReturnType(volumeObjects) === 'Array') {
									for (var volumeIndex in volumeObjects) {
										if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sub') {
											volume_m3sub += parseFloat(volumeObjects[volumeIndex]['_']);
										} else if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
											volume_m3sob += parseFloat(volumeObjects[volumeIndex]['_']);
										}
									}
								}
								//console.log(harvesterDataObjects.NumberOfHarvestedStems);
							}
						} else if (getJsonReturnType(harvesterDataObjects) === 'Array') {
							//console.log('HARVESTER DATA: Harvester data object is an ARRAY');
							//console.log('HARVESTER DATA: Harvester data arrray length ' + harvesterDataObjects.length);
							for (var hdObjectIndex in harvesterDataObjects) {
								//console.log('HARVESTER DATA: Iterating harvester data object ' + hdObjectIndex);
								//console.log('HARVESTER DATA: Processing category test. Expecting ' + uniqueProcessingCategories[pcIndex] + ', is ' + harvesterDataObjects[hdObjectIndex].ProcessingCategory);
								//console.log('HARVESTER DATA: Species group key test. Expecting ' + uniqueSpeciesGroupKeys[index] + ', is ' + getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex]));
								if (uniqueProcessingCategories[pcIndex] === harvesterDataObjects[hdObjectIndex].ProcessingCategory && uniqueSpeciesGroupKeys[index] === getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObjects[hdObjectIndex])) {
									//console.log('HARVESTER DATA: Processing category match AND species group key match');
									//console.log('HARVESTER DATA: Getting number of harvested stems for calculation. Got ' + harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
									numberOfHarvestedStems += parseInt(harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
									volumeObjects = harvesterDataObjects[hdObjectIndex].TotalVolumeOfHarvestedLogs;
									if (getJsonReturnType(volumeObjects) === 'Object') {
										if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sub') {
											volume_m3sub += parseFloat(volumeObjects['_']);
										} else if (volumeObjects.$.harvestedLogsVolumeCategory === 'm3sob') {
											volume_m3sob += parseFloat(volumeObjects['_']);
										}
									} else if (getJsonReturnType(volumeObjects) === 'Array') {
										for (var volumeIndex in volumeObjects) {
											if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sub') {
												volume_m3sub += parseFloat(volumeObjects[volumeIndex]['_']);
											} else if (volumeObjects[volumeIndex].$.harvestedLogsVolumeCategory === 'm3sob') {
												volume_m3sob += parseFloat(volumeObjects[volumeIndex]['_']);
											}
										}
									}
									//console.log(harvesterDataObjects[hdObjectIndex].NumberOfHarvestedStems);
								}
							}
						}
					}
				}
				
			}
			//aggregateArrayItem.push(uniqueSpeciesGroupKeys[index] + '::' + numberOfHarvestedStems + '::' + volume_m3sub.toFixed(3) + '::' + volume_m3sob.toFixed(3));
			aggregateJsonItem += '{ "speciesGroupKey":"' + uniqueSpeciesGroupKeys[index] + '", "numberOfHarvestedStems":' + numberOfHarvestedStems + ', "m3sub":' + volume_m3sub.toFixed(3) + ', "m3sob":' + volume_m3sob.toFixed(3) + '}, ';
			numberOfHarvestedStems = 0;
			volume_m3sub = 0;
			volume_m3sob = 0;
		}
		
		aggregateJson += aggregateJsonItem + ', ';
		aggregateJsonItem = '';
	}
	aggregateJson += '] } ] }';
	aggregateJson = aggregateJson.replace('}, , ] } ] }','} ] } ] }');
	aggregateJson = aggregateJson.replace('[] } ] }','[] }');
	console.log(aggregateJson);
	//console.log(aggregateArrayItem);
	return aggregateJson;

}

function getIndividualMachineWorkTime_otherMachineData_harvesterDataObjects(otherMachineDataObject) {
	return otherMachineDataObject.HarvesterData;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_numberOfHarvestedStems(harvesterDataObject) {
	return harvesterDataObject.NumberOfHarvestedStems;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_processingCategory(harvesterDataObject) {
	return harvesterDataObject.ProcessingCategory;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_speciesGroupKey(harvesterDataObject) {
	return harvesterDataObject.SpeciesGroupKey;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_totalVolumeOfHarvestedLogsObjects(otherMachineDataObject) {
	return otherMachineDataObject.TotalVolumeOfHarvestedLogs;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_totalVolumeOfHarvestedLogs_harvestedLogsVolumeCategory(totalVolumeOfHarvestedLogsObject) {
	return totalVolumeOfHarvestedLogsObject.$.harvestedLogsVolumeCategory;
}

function getIndividualMachineWorkTime_otherMachineData_harvesterData_totalVolumeOfHarvestedLogs_harvestedLogsVolume(totalVolumeOfHarvestedLogsObject) {
	return totalVolumeOfHarvestedLogsObject['_'];
}

function getIndividualMachineWorkTime_getUniqueMachineRunTimeCategories(individualMachineWorkTimeObjects, operatorKey) {
	
	var machineRunTimeCategories = [];
	var uniqueMachineRunTimeCategories = [];
	
	if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
		if (individualMachineWorkTimeObjects.OperatorKey === operatorKey) {
			if (individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory) {
				machineRunTimeCategories.push(individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory);
			}
		}
	} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
		for (var objectIndex in individualMachineWorkTimeObjects) {
			if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey) {
				if (individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory) {
					machineRunTimeCategories.push(individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory);
				}
			}
		}
	}
	
	uniqueMachineRunTimeCategories = uniq(machineRunTimeCategories);
	
	return uniqueMachineRunTimeCategories;
	
}

function getIndividualMachineWorkTime_getRunTimeCategoryMonitoringTimeLength(individualMachineWorkTimeObjects, uniqueMachineRunTimeCategories, operatorKey) {
	
	var machineRunTimeCategory;
	var sumOfMonitoringTimes = 0;
	var categoryString;
	var categoryAndSumArray = [];
	for (var index in uniqueMachineRunTimeCategories) {
		machineRunTimeCategory = uniqueMachineRunTimeCategories[index];
		categoryString = uniqueMachineRunTimeCategories[index];
		if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
			if (individualMachineWorkTimeObjects.OperatorKey === operatorKey && individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
				sumOfMonitoringTimes = parseInt(individualMachineWorkTimeObjects.MonitoringTimeLength);
			}
		} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
			for (var objectIndex in individualMachineWorkTimeObjects) {
				if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey && individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
					sumOfMonitoringTimes += parseInt(individualMachineWorkTimeObjects[objectIndex].MonitoringTimeLength);
				}
			}
		}
		categoryAndSumArray.push(categoryString + '=' + sumOfMonitoringTimes);
	}
	return categoryAndSumArray;
	
}

function getIndividualMachineWorkTime_getRunTimeCategoryEngineTimeSum(individualMachineWorkTimeObjects, uniqueMachineRunTimeCategories, operatorKey) {
	
	var machineRunTimeCategory;
	var sumOfEngineTime = 0;
	var categoryString;
	var categoryAndSumArray = [];
	for (var index in uniqueMachineRunTimeCategories) {
		machineRunTimeCategory = uniqueMachineRunTimeCategories[index];
		categoryString = uniqueMachineRunTimeCategories[index];
		if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
			if (individualMachineWorkTimeObjects.OperatorKey === operatorKey && individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
				sumOfEngineTime = parseInt(individualMachineWorkTimeObjects.OtherMachineData.EngineTime);
			}
		} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
			for (var objectIndex in individualMachineWorkTimeObjects) {
				if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey && individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
					sumOfEngineTime += parseInt(individualMachineWorkTimeObjects[objectIndex].OtherMachineData.EngineTime);
				}
			}
		}
		categoryAndSumArray.push(categoryString + '=' + sumOfEngineTime);
	}
	return categoryAndSumArray;
	
}

function getIndividualMachineWorkTime_getRunTimeCategoryDrivenDistanceSum(individualMachineWorkTimeObjects, uniqueMachineRunTimeCategories, operatorKey) {
	
	var machineRunTimeCategory;
	var sumOfDrivenDistance = 0;
	var categoryString;
	var categoryAndSumArray = [];
	for (var index in uniqueMachineRunTimeCategories) {
		machineRunTimeCategory = uniqueMachineRunTimeCategories[index];
		categoryString = uniqueMachineRunTimeCategories[index];
		if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
			if (individualMachineWorkTimeObjects.OperatorKey === operatorKey && individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
				sumOfDrivenDistance = parseInt(individualMachineWorkTimeObjects.OtherMachineData.DrivenDistance);
			}
		} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
			for (var objectIndex in individualMachineWorkTimeObjects) {
				if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey && individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
					sumOfDrivenDistance += parseInt(individualMachineWorkTimeObjects[objectIndex].OtherMachineData.DrivenDistance);
				}
			}
		}
		categoryAndSumArray.push(categoryString + '=' + sumOfDrivenDistance);
	}
	return categoryAndSumArray;
	
}

function getIndividualMachineWorkTime_getRunTimeCategoryFuelConsumptionSum(individualMachineWorkTimeObjects, uniqueMachineRunTimeCategories, operatorKey) {
	
	var machineRunTimeCategory;
	var sumOfFuelConsumption = 0;
	var categoryString;
	var categoryAndSumArray = [];
	for (var index in uniqueMachineRunTimeCategories) {
		machineRunTimeCategory = uniqueMachineRunTimeCategories[index];
		categoryString = uniqueMachineRunTimeCategories[index];
		if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Object') {
			if (individualMachineWorkTimeObjects.OperatorKey === operatorKey && individualMachineWorkTimeObjects.IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
				sumOfFuelConsumption = parseInt(individualMachineWorkTimeObjects.OtherMachineData.FuelConsumption);
			}
		} else if (getJsonReturnType(individualMachineWorkTimeObjects) === 'Array') {
			for (var objectIndex in individualMachineWorkTimeObjects) {
				if (individualMachineWorkTimeObjects[objectIndex].OperatorKey === operatorKey && individualMachineWorkTimeObjects[objectIndex].IndividualMachineRunTimeCategory === uniqueMachineRunTimeCategories[index]) {
					sumOfFuelConsumption += parseInt(individualMachineWorkTimeObjects[objectIndex].OtherMachineData.FuelConsumption);
				}
			}
		}
		categoryAndSumArray.push(categoryString + '=' + sumOfFuelConsumption);
	}
	return categoryAndSumArray;
	
}

function FUNCTION_iterateOperatorWorkTimeObjects(payload, operatorKey) {
	var workTimeObject = OPERATIONAL_MONITORING_MACHINE_getOperatorWorkTimeObject(payload);
	if (getJsonReturnType(workTimeObject) === 'Object') {
		if (workTimeObject.OperatorKey === operatorKey) {
			console.log(operatorKey);
			console.log(workTimeObject.$.OperatorWorkTimeCategory);
			console.log(workTimeObject.MonitoringStartTime);
			console.log(workTimeObject.MonitoringTimeLength);
			console.log(getTimePlusSeconds(getTime(workTimeObject.MonitoringStartTime), workTimeObject.MonitoringTimeLength));
		}
	} else if (getJsonReturnType(workTimeObject) === 'Array') {
		for (var objectIndex in workTimeObject) {
			if (workTimeObject[objectIndex].OperatorKey === operatorKey) {
				console.log(operatorKey);
				console.log(workTimeObject[objectIndex].$.OperatorWorkTimeCategory);
				console.log(workTimeObject[objectIndex].MonitoringStartTime);
				console.log(workTimeObject[objectIndex].MonitoringTimeLength);
				console.log(getTimePlusSeconds(getTime(workTimeObject[objectIndex].MonitoringStartTime), parseInt(workTimeObject[objectIndex].MonitoringTimeLength)));
			}
		}
	}
}

function OPERATIONAL_MONITORING_MACHINE_getOperatorWorkTimeObject(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	return jsonObject.payload['#document'].OperationalMonitoring.Machine.OperatorWorkTime;
}

function FUNCTION_getOperatorWorkTimeSum(JSONObject, operatorKey) {
	
	var sum = 0;
	
	if (getJsonReturnType(JSONObject) === 'Object') {
		
		if (JSONObject.OperatorKey === operatorKey) {
			sum = JSONObject.MonitoringTimeLength;
		}
		
	} else if (getJsonReturnType(JSONObject) === 'Array') {
	
		var objectInArray;
		for (var objectIndex in JSONObject) {
			if (JSONObject[objectIndex].OperatorKey === operatorKey) {
				sum += parseInt(JSONObject[objectIndex].MonitoringTimeLength);
			}
		}
			
	} else { 
		
		sum = 0;
		
	}
	
	return sum;
}

function FUNCTION_getRecodedWorktimeCategory (worktimeCategory) {
	if (worktimeCategory === 'MachineWorkTime') {
		return 'Machine work time';
	} else {
		return worktimeCategory;
	}
}

function FUNCTION_getUniqueOperatorWorkTimeCategories(JSONObject, operatorKey) {
	
	var operatorWorkTimeCategory = [];
	
	if (getJsonReturnType(JSONObject) === 'Object') {
		if (JSONObject.OperatorKey === operatorKey) {
			operatorWorkTimeCategory.push(JSONObject.$.OperatorWorkTimeCategory);
		}
	} else if (getJsonReturnType(JSONObject) === 'Array') {
		var objectInArray;
		for (var objectIndex in JSONObject) {
			if (JSONObject[objectIndex].OperatorKey === operatorKey) {
				operatorWorkTimeCategory.push(JSONObject[objectIndex].$.OperatorWorkTimeCategory);
			}
		}
			
	} else { 
		
		operatorWorkTimeCategory = [];
		
	}
	
	return uniq(operatorWorkTimeCategory);
	
}

function FUNCTION_getOperatorWorkTimeSumByWorkTimeCategory(JSONObject, operatorKey, workTimeCategory) {
	
	var sum = 0;
	
	if (getJsonReturnType(JSONObject) === 'Object') {
		
		if (JSONObject.OperatorKey === operatorKey && JSONObject.$.OperatorWorkTimeCategory) {
			sum = JSONObject.MonitoringTimeLength;
		}
		
	} else if (getJsonReturnType(JSONObject) === 'Array') {
	
		var objectInArray;
		for (var objectIndex in JSONObject) {
			if (JSONObject[objectIndex].OperatorKey === operatorKey && JSONObject[objectIndex].$.OperatorWorkTimeCategory === workTimeCategory) {
				sum += parseInt(JSONObject[objectIndex].MonitoringTimeLength);
			}
		}
			
	} else { 
		
		sum = 0;
		
	}
	
	return sum;
	
}

function OPERATIONAL_MONITORING_MACHINE_getOperatorShiftDefinition(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	return jsonObject.payload['#document'].OperationalMonitoring.Machine.OperatorShiftDefinition;
}

function FUNCTION_getOperatorShiftCount(shiftDefinitionObject, operatorKey) {
	var shiftCount = 0;
	if (getJsonReturnType(shiftDefinitionObject) === 'Object') {
		if (shiftDefinitionObject.OperatorKey === operatorKey) {
			shiftCount = 1;
		}
	} else if (getJsonReturnType(shiftDefinitionObject) === 'Array') {
		for (var shiftIndex in shiftDefinitionObject) {
			if (shiftDefinitionObject[shiftIndex].OperatorKey === operatorKey) {
				shiftCount += 1;
			}
		}
	} else {
	
	}
	return shiftCount;
}

function FUNCTION_getOperatorShiftsDuration(JSONObject, operatorKey) {

	var sum = 0;
	
	if (getJsonReturnType(JSONObject) === 'Object') {
		
		if (JSONObject.OperatorKey === operatorKey) {
			sum = parseInt(FUNCTION_getOperatorShiftDurationInMilliseconds(JSONObject));
		}
		
	} else if (getJsonReturnType(JSONObject) === 'Array') {
	
		var objectInArray;
		for (var objectIndex in JSONObject) {
			if (JSONObject[objectIndex].OperatorKey === operatorKey) {
				sum += parseInt(FUNCTION_getOperatorShiftDurationInMilliseconds(JSONObject[objectIndex]));
			}
		}
			
	} else { 
		
		sum = 0;
		
	}
	
	return sum;

}

function FUNCTION_getOperatorShiftObjectOperatorKey(operatorShiftObject) {
	return operatorShiftObject.OperatorKey;
}

function FUNCTION_getOperatorShiftKey(operatorShiftObject) {
	return operatorShiftObject.ShifKey;
}

function FUNCTION_getOperatorShiftCategory(operatorShiftObject) {
	return operatorShiftObject.ShiftCategory;
}

function FUNCTION_getOperatorShiftStartTime(operatorShiftObject) {
	return operatorShiftObject.ShiftStartTime;
}

function FUNCTION_getOperatorShiftEndTime(operatorShiftObject) {
	return operatorShiftObject.ShiftEndTime;
}

function FUNCTION_getOperatorShiftDurationInMilliseconds(operatorShiftObject) {
	var startDateTime = FUNCTION_getOperatorShiftStartTime(operatorShiftObject);
	var startYear = startDateTime.substring(0,4);
	var startMonth = startDateTime.substring(5,7);
	var startDay = startDateTime.substring(8,10);
	var startHour = startDateTime.substring(11,13);
	var startMinute = startDateTime.substring(14,16);
	var startSecond = startDateTime.substring(17,19);
	var startMillisecond = startDateTime.substring(20,23);
	if (startDateTime.length < 29) { startMillisecond = '000'; }
	//console.log(startYear + ' ' + startMonth + ' ' + startDay + ' ' + startHour + ' ' + startMinute + ' ' + startSecond + ' ' + startMillisecond);
	var startDateTimeInMilliseconds = new Date(startYear, startMonth, startDay, startHour, startMinute, startSecond, startMillisecond);
	var endDateTime = FUNCTION_getOperatorShiftEndTime(operatorShiftObject);
	var endYear = endDateTime.substring(0,4);
	var endMonth = endDateTime.substring(5,7);
	var endDay = endDateTime.substring(8,10);
	var endHour = endDateTime.substring(11,13);
	var endMinute = endDateTime.substring(14,16);
	var endSecond = endDateTime.substring(17,19);
	var endMillisecond = endDateTime.substring(20,23);
	if (endDateTime.length < 29) { endMillisecond = '000'; }
	//console.log(endYear + ' ' + endMonth + ' ' + endDay + ' ' + endHour + ' ' + endMinute + ' ' + endSecond + ' ' + endMillisecond);
	var endDateTimeInMilliseconds = new Date(endYear, endMonth, endDay, endHour, endMinute, endSecond, endMillisecond);
	return (endDateTimeInMilliseconds-startDateTimeInMilliseconds);
	
}

function OPERATIONAL_MONITORING_MACHINE_getOperatorLoginTime(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	return jsonObject.payload['#document'].OperationalMonitoring.Machine.OperatorLoginTime;
}

function FUNCTION_getOperatorLoginTimeCount(loginTimeObject, operatorKey) {
	var loginTimeCount = 0;
	if (getJsonReturnType(loginTimeObject) === 'Object') {
		if (loginTimeObject.OperatorKey === operatorKey) {
			loginTimeCount = 1;
		}
	} else if (getJsonReturnType(loginTimeObject) === 'Array') {
		for (var logintimeIndex in loginTimeObject) {
			if (loginTimeObject[logintimeIndex].OperatorKey === operatorKey) {
				loginTimeCount += 1;
			}
		}
	} else {
	
	}
	return loginTimeCount;
}

function FUNCTION_getOperatorLoginTimeSum(loginTimeObject, operatorKey) {
	var loginTimeSum = 0;
	if (getJsonReturnType(loginTimeObject) === 'Object') {
		if (loginTimeObject.OperatorKey === operatorKey) {
			loginTimeSum = parseInt(loginTimeObject.MonitoringTimeLength);
		}
	} else if (getJsonReturnType(loginTimeObject) === 'Array') {
		for (var logintimeIndex in loginTimeObject) {
			if (loginTimeObject[logintimeIndex].OperatorKey === operatorKey) {
				loginTimeSum += parseInt(loginTimeObject[logintimeIndex].MonitoringTimeLength);
			}
		}
	} else {
	
	}
	return loginTimeSum;
}

function FUNCTION_getOperatorLoginTimeObjectOperatorKey(operatorLoginTimeObject) {
	return operatorLoginTimeObject.OperatorKey;
}

function FUNCTION_getOperatorLoginStartTime(operatorLoginTimeObject) {
	return operatorLoginTimeObject.MonitoringStartTime;
}

function FUNCTION_getOperatorLoginTimeLength(operatorLoginTimeObject) {
	return operatorLoginTimeObject.MonitoringTimeLength;
}

function OPERATIONAL_MONITORING_MACHINE_getMachineWorkTimeObjects(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	//console.log(jsonObject);
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime) {
		//console.log('individual');
		return jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
	} else if (jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime) {
		//console.log('combined');
		return jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
	} else {
		//console.log('nothing');
		return '';
	}
}

function OPERATIONAL_MONITORING_MACHINE_getOperatorMachineWorkTimeObjects(momAsJSON, operatorKey) {

}

function FUNCTION_getMachineWorkTimeObjectType(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	if (jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime) {
		return 'COMBINED';
	} else if (jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime) {
		return 'INDIVIDUAL';
	} else {
		return 'UNDEFINED';
	}
}

/*
 *	DEPRECATED
 */
function OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjects(momAsJSON) {
	
	var jsonObject = JSON.parse(momAsJSON);
	var workTimeObjectType = OPERATIONAL_MONITORING_MACHINE_getWorkTimeObjectType(momAsJSON);
	var workTimeObjectKey;
	
	var workTimeObject = '{"workTimeObjectType":"';
	
	switch(workTimeObjectType) {
		case "COMBINED":
			//workTimeObjectKey = 'CombinedMachineWorkTime';
			workTimeObject += 'COMBINED", "payload":';
			workTimeObject += JSON.stringify(jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime);
			workTimeObject += '}';
			break;
		case "INDIVIDUAL":
			//workTimeObjectKey = "IndividualMachineWorkTime";
			workTimeObject += 'INDIVIDUAL", "payload":';
			workTimeObject += JSON.stringify(jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime);
			workTimeObject += '}';
			break;
		default:
			return;
	}
	
	//console.log(workTimeObject);
	
	return workTimeObject;
	
	//var jsonObject = JSON.parse(momAsJSON);
	//var workTimeObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine[workTimeObjectKey];
	//var workTimeObjectsType = getJsonReturnType(workTimeObjects);
	//console.log(workTimeObjectKey);
	//console.log(workTimeObjectsType);
	//console.log(workTimeObjects);
}

function OPERATIONAL_MONITORING_MACHINE_getWhateverObject(momAsJSON, objectKey) {
	var jsonObject = JSON.parse(momAsJSON);
	var whateverObject = jsonObject.payload['#document'].OperationalMonitoring.Machine[objectKey];
	return whateverObject;
}

function GENERAL_getValue(jsonObject, keyForKeyValuePair) {
	var valueFromKeyValuePair = jsonObject[keyForKeyValuePair];
	return valueFromKeyValuePair;
}

function OPERATIONAL_MONITORING_MACHINE_getCombinedWorkTimeObject(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var individualWorkTimeObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.CombinedMachineWorkTime;
}

function OPERATIONAL_MONITORING_MACHINE_getIndividualWorkTimeObject(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var individualWorkTimeObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualMachineWorkTime;
}

function OPERATIONAL_MONITORING_MACHINE_getObjectDefinition(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var objectDefinition = jsonObject.payload['#document'].OperationalMonitoring.Machine['ObjectDefinition'];
	return objectDefinition;
}




	
function OPERATIONAL_MONITORING_MACHINE_OBJECT_DEFINITION_getForestOwner(momAsJSON) {

}

function OPERATIONAL_MONITORING_MACHINE_getSpeciesGroupDefinitionObjects(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var speciesGroupDefinitionObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.SpeciesGroupDefinition;
	var speciesGroupName;
	var speciesGroupKey;
	var dbhh;
	var response = '[';
	for (var speciesGroupObject in speciesGroupDefinitionObjects) {
		speciesGroupName = speciesGroupDefinitionObjects[speciesGroupObject].SpeciesGroupName;
		speciesGroupKey = speciesGroupDefinitionObjects[speciesGroupObject].SpeciesGroupKey;
		dbhh = speciesGroupDefinitionObjects[speciesGroupObject].DBHHeight['_'];
		response += '{"groupkey":"' + speciesGroupKey + '", "groupname":"' + speciesGroupName + '", "dbhh":"' + dbhh + '"}, ';
	}
	response += ']';
	response = response.replace('}, ]','}]');
	return response;
}

function OPERATIONAL_MONITORING_MACHINE_getSpeciesGroupName(momAsJSON, speciesGroupKey) {
	var jsonObject = JSON.parse(momAsJSON);
	var speciesGroupDefinitionObjects = jsonObject.payload['#document'].OperationalMonitoring.Machine.SpeciesGroupDefinition;
	var speciesGroupName;
	for (var speciesGroupObject in speciesGroupDefinitionObjects) {
		if (speciesGroupKey === speciesGroupDefinitionObjects[speciesGroupObject].SpeciesGroupKey) {
			speciesGroupName = speciesGroupDefinitionObjects [speciesGroupObject].SpeciesGroupName;
			return speciesGroupName;
		}
	}
	return 'No name for code ' + speciesGroupKey;
}

function OPERATIONAL_MONITORING_ROOT_getValue(momAsJSON, momKey) {
	var jsonObject = JSON.parse(momAsJSON);
	var momValue = jsonObject.payload['#document'].OperationalMonitoring['$'][momKey];
	//console.log(momKey + ': ' + momValue);
	return momValue;
}

function OPERATIONAL_MONITORING_HEADER_getValue(momAsJSON, momKey) {
	var jsonObject = JSON.parse(momAsJSON);
	var momValue = jsonObject.payload['#document'].OperationalMonitoring.OperationalMonitoringHeader[momKey];
	//console.log(momKey + ': ' + momValue);
	return momValue;
}



function OPERATIONAL_MONITORING_MACHINE_getMonitoringSettingsCount(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var monitoringSettingsObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.MonitoringSettings;
	var objectType = getJsonReturnType(monitoringSettingsObject);
	
	switch (objectType) {
		case 'Object':
			return 1;
			break;
		case 'Array':
			return monitoringSettingsObject.length;
			break;
		default:
			return 0;
	}
	
}



function OPERATIONAL_MONITORING_MACHINE_MONITORING_SETTINGS_getMonitoringSettingsKey(monitoringSettingsObject) {
	return monitoringSettingsObject.MonitoringSettingsKey;
}





function OPERATIONAL_MONITORING_MACHINE_getShortDownTimes(momAsJSON) {
	var jsonObject = JSON.parse(momAsJSON);
	var typeDefinition = 'FrequencyShortDownTimes';
	var shortDownTimeObjectWithTypeDefinition = '{';
	var shortDownTimesObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.FrequencyShortDownTimes;
	if (getJsonReturnType(shortDownTimesObject) === 'undefined') {
		shortDownTimesObject = jsonObject.payload['#document'].OperationalMonitoring.Machine.IndividualShortDownTime;
		typeDefinition = 'IndividualShortDownTime';
	}
	
	//console.log(getJsonReturnType(shortDownTimesObject));
	
	var shortDownTimeArray = [];
	var shortDownTimeObject;
	var shortDownTimeObjectWithTypeDefinition;
	
	if (getJsonReturnType(shortDownTimesObject) === 'Object') {
		shortDownTimeObject = shortDownTimesObject;
		shortDownTimeObjectWithTypeDefinition = '{"objectTypeDefinition":"' + typeDefinition + '", "payload":' + JSON.stringify(shortDownTimeObject) + '}';
		shortDownTimeArray.push(shortDownTimeObjectWithTypeDefinition);
	} else {
		for (var object in shortDownTimesObject) {
			shortDownTimeObject = shortDownTimesObject[object];
			//console.log(shortDownTimeObject);
			shortDownTimeObjectWithTypeDefinition = '{"objectTypeDefinition":"' + typeDefinition + '", "payload":' + JSON.stringify(shortDownTimeObject) + '}';
			shortDownTimeArray.push(shortDownTimeObjectWithTypeDefinition);
		}
	}
	return shortDownTimeArray;
	
}

function OPERATIONAL_MONITORING_MACHINE_POSITION_getValue(momAsJSON, momKey) {
	var jsonObject = JSON.parse(momAsJSON);
	if(getJsonReturnType(jsonObject.payload['#document'].OperationalMonitoring.Machine.Extension) === 'undefined' || getJsonReturnType(jsonObject.payload['#document'].OperationalMonitoring.Machine.Extension.MachinePosition) === 'undefined') {
		//console.log(momKey + ': -');
		return '-';
	} else {
		var momValue = jsonObject.payload['#document'].OperationalMonitoring.Machine.Extension.MachinePosition[momKey]['_'];
		//console.log(momKey + ': ' + momValue);
		return momValue;
	}
}



function machineBaseManufacturer (momAsJSON) {

	var jsonObject = JSON.parse(momAsJSON);
	var machineBaseManufacturerValue = jsonObject['#document'].OperationalMonitoring.Machine.MachineBaseManufacturer;
	alert(machineBaseManufacturerValue);

}

function momParser(momAsJSON) {

	machineBaseManufacturer(momAsJSON);
	
}